/***************************************************************************
 $RCSfile: loader.cpp,v $
                             -------------------
    cvs         : $Id: loader.cpp,v 1.7 2004/01/19 18:53:49 aquamaniac Exp $
    begin       : Sat Jun 08 2002
    copyright   : (C) 2001 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "loader.h"
#include <hbcistring.h>
#include <assert.h>
#include <gwenhywfar/debug.h>

namespace HBCI {

Loader::Loader(API *hbci):_hbci(hbci) {
}


Loader::~Loader() {
}


Error Loader::saveCustomer(Pointer<Customer> c,
			   GWEN_DB_NODE *cfg) {
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "id",c.ref().custId().c_str());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "name",c.ref().name().c_str());
  return Error();
}


Error Loader::saveUser(Pointer<User> c,
		       GWEN_DB_NODE *cfg) {
  list<Pointer<Customer> > custs;
  list<Pointer<Customer> >::iterator cit;
  GWEN_DB_NODE *cfg1, *cfg2;
  Error err;
  int nextGroupNr;

  //fprintf(stderr,"Saving user %s\n",c.ref().userId().c_str());

  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "id",c.ref().userId().c_str());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "name",c.ref().name().c_str());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "systemid",c.ref().systemId().c_str());

  // save medium
  if (c.ref().medium().isValid()) {
    cfg1=GWEN_DB_GetGroup(cfg, GWEN_DB_FLAGS_DEFAULT, "medium");
    if (!cfg1)
      return Error("Loader::saveUser()",
		   ERROR_LEVEL_INTERNAL,
		   0,
		   ERROR_ADVISE_ABORT,
		   "Could not create group",
		   "medium");
    err=_hbci->mediumToConfig(c.ref().medium(),cfg1);
    if (!err.isOk())
      return err;

  } // if medium

  // save customers
  custs=c.ref().customers();
  if (!custs.empty()) {
    cfg1=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT,
                          "customer");
    if (!cfg1)
      return Error("Loader::saveUser()",
		   ERROR_LEVEL_INTERNAL,
		   0,
		   ERROR_ADVISE_ABORT,
		   "Could not create group",
		   "customer");
    nextGroupNr=0;
    for (cit=custs.begin(); cit!=custs.end(); cit++) {
      cfg2=GWEN_DB_GetGroup(cfg1,
                            GWEN_DB_FLAGS_DEFAULT,
                            String::num2string(nextGroupNr++).c_str());
      if (!cfg2)
	return Error("Loader::saveUser()",
		     ERROR_LEVEL_INTERNAL,
		     0,
		     ERROR_ADVISE_ABORT,
		     "Could not create group",
		     (*cit).ref().custId());
      err=saveCustomer(*cit,cfg2);
      if (!err.isOk())
	return err;
    } // for
  }

  return Error();
}




Error Loader::saveBank(Pointer<Bank> b,
		       GWEN_DB_NODE *cfg,
		       unsigned int flags) {
  GWEN_DB_NODE *cfg1;
  Error err;
  list<Pointer<User> > custs;
  list<Pointer<User> >::const_iterator cit;
  list<int>::const_iterator vit;
  int nextGroupNr;
  Pointer<MessageEngine> mngin;
  Pointer<Transporter> xport;

  GWEN_DB_SetIntValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                      "hbciversion",b.ref().hbciVersion());
  GWEN_DB_SetIntValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                      "country", b.ref().country());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "code",b.ref().bankCode().c_str());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "addr",b.ref().address().c_str());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "port",b.ref().port().c_str());
  GWEN_DB_SetCharValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                       "name",b.ref().name().c_str());

  for (vit=b.ref().hbciVersions().begin();
       vit!=b.ref().hbciVersions().end();
       vit++) {
    GWEN_DB_SetIntValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                        "hbciversions", *vit);
  } // for

  // save customers
  custs=b.ref().users();
  nextGroupNr=0;
  for (cit=custs.begin(); cit!=custs.end(); cit++) {
    cfg1=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT,
                          string("user/"
                                 +String::num2string(nextGroupNr++)).c_str());
    if (!cfg1)
      return Error("Loader::saveBank()",
		   ERROR_LEVEL_INTERNAL,
		   0,
		   ERROR_ADVISE_ABORT,
		   "Could not create group",
		   "customer");
    err=saveUser(*cit,cfg1);
    if (!err.isOk())
      return err;
  } // for


  // save message engine
  mngin=b.ref().messageEngine();
  if (mngin.isValid()) {
    cfg1=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT,
                          "engine");
    assert(cfg1);
    err=_hbci->engineToConfig(mngin, cfg1);
    if (!err.isOk())
      return Error("Loader::saveBank", err);
  }

  // save transporter
  xport=b.ref().transporter();
  if (xport.isValid()) {
    cfg1=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT,
                          "transport");
    assert(cfg1);
    err=_hbci->transporterToConfig(xport, cfg1);
    if (!err.isOk())
      return Error("Loader::saveBank", err);
  }

  // done
  return Error();
}


Error Loader::saveAll(GWEN_DB_NODE *cfg,
		      unsigned int flags) {
  list<Pointer<Bank> > banks;
  list<Pointer<Bank> >::const_iterator bit;
  GWEN_DB_NODE *cfg1;
  Error err;
  int nextGroupNr;

  GWEN_DB_SetIntValue(cfg, GWEN_DB_FLAGS_DEFAULT,
                      "nextjobid",Outbox::nextId());

  banks=_hbci->banks();
  nextGroupNr=0;
  for (bit=banks.begin(); bit!=banks.end(); bit++) {
    if ((*bit).ref().bankCode().empty())
      return Error("Loader::saveBank()",
		   ERROR_LEVEL_INTERNAL,
		   0,
		   ERROR_ADVISE_ABORT,
		   "bank has no id !");
    cfg1=GWEN_DB_GetGroup(cfg,
                          GWEN_DB_FLAGS_DEFAULT,
                          string("bank/"
                                 +String::num2string(nextGroupNr++)).c_str());
    if (!cfg1)
      return Error("Loader::saveAll()",
		   ERROR_LEVEL_INTERNAL,
		   0,
		   ERROR_ADVISE_ABORT,
		   "Could not create group",
		   "bank/"+String::num2string(nextGroupNr-1));
    err=saveBank(*bit,cfg1);
    if (!err.isOk())
      return err;
  } // for

  return Error();
}



Error Loader::loadCustomer(Pointer<User> u,
			   Pointer<Customer> &cu,
			   GWEN_DB_NODE *cfg) {
  string id;
  string custname;

  id=GWEN_DB_GetCharValue(cfg, "id",0, "");
  if (id.empty())
    return Error("Loader::loadCustomer()",
		 ERROR_LEVEL_INTERNAL,
		 0,
		 ERROR_ADVISE_ABORT,
		 "customer id is empty");
  custname=GWEN_DB_GetCharValue(cfg, "name",0, "");

  cu=_hbci->customerFactory(u, id, custname);
  return Error();
}


Error Loader::loadUser(Pointer<Bank> b,
		       GWEN_DB_NODE *cfg) {
  Pointer<User> u;
  Pointer<Customer> cu;
  Pointer<Medium> m;
  Error err;
  string mname;
  GWEN_DB_NODE *cfg1;
  string id;
  string username;
  string t;

  // sample all data we need
  id=GWEN_DB_GetCharValue(cfg, "id",0, "");
  if (id.empty())
    return Error("Loader::loadUser()",
		 ERROR_LEVEL_INTERNAL,
		 0,
		 ERROR_ADVISE_ABORT,
		 "customer with empty id");
  username=GWEN_DB_GetCharValue(cfg, "name",0, "");

  // check for "[medium]" group
  cfg1=GWEN_DB_GetGroup(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "medium");
  if (!cfg1) {
    // group does not exist, so simply use the user's group
    cfg1=cfg;
  }

  // look for a medium of that name
  DBG_INFO(0,"Setting up medium");
  mname=GWEN_DB_GetCharValue(cfg1, "mediumname",0, "");
  DBG_INFO(0, "Searching for medium");
  if (!mname.empty())
    m=_hbci->findMedium(mname);

  // check if the medium already exists
  if (!m.isValid()) {
    // it does not, so create it
    try {
      DBG_INFO(0, "Creating medium from config");
      m=_hbci->mediumFromConfig(cfg1);
    }
    catch (Error xerr) {
      return Error("Loader::loadUser", xerr);
    }
  }

  // create user
  DBG_INFO(0, "Creating user");
  u=_hbci->userFactory(b, m, id, username);
  u.ref().setSystemId(GWEN_DB_GetCharValue(cfg, "systemid",0, "0"));
  m.ref().setOwner(u);

  // add user to the bank
  DBG_INFO(0, "Adding user");
  b.ref().addUser(u);

  // load all customers
  cfg1=GWEN_DB_GetGroup(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "customer");
  if (cfg1) {
    cfg1=GWEN_DB_GetFirstGroup(cfg1);
    while(cfg1) {
      DBG_INFO(0, "Checking group \%s\"",
               GWEN_DB_GroupName(cfg1));
      err=loadCustomer(u,cu,cfg1);
      if (!err.isOk())
	return err;
      // add customer to list
      DBG_INFO(0, "Adding customer");
      u.ref().addCustomer(cu);
      cfg1=GWEN_DB_GetNextGroup(cfg1);
    } // while there are customers
  } // if group exists

  // check whether there is at least one customer
  if (u.ref().customers().size()==0) {
    // there is none, so create this customer
    DBG_INFO(0, "Creating default customer for user \"%s\"",
             u.ref().userId().c_str());
    cu =_hbci->customerFactory(u, u.ref().userId(), u.ref().name());
    u.ref().addCustomer(cu);
  }

  return Error();
}


Error Loader::loadBank(GWEN_DB_NODE *cfg, unsigned int flags) {
  GWEN_DB_NODE *cfg1;
  Error err;
  Pointer<Bank> bank;
  Pointer<MessageEngine> mngin;
  Pointer<Transporter> xport;
  int i;
  int v;

  // Create new BankImpl object
  bank=_hbci->bankFactory(GWEN_DB_GetIntValue(cfg, "country",0, 280),
                          GWEN_DB_GetCharValue(cfg, "code",0, ""),
                          GWEN_DB_GetCharValue(cfg, "addr", 0, ""),
                          GWEN_DB_GetCharValue(cfg, "port", 0, ""),
                          GWEN_DB_GetIntValue(cfg, "hbciversion",0,0),
                          GWEN_DB_GetCharValue(cfg, "name", 0, ""));

  for (i=0; i<16; i++) {
    v=GWEN_DB_GetIntValue(cfg, "hbciversions", i, -1);
    if (v==-1)
      break;
    bank.ref().addHbciVersion(v);
  } // for


  // load message engine
  cfg1=GWEN_DB_GetGroup(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "engine");
  if (cfg1) {
    try {
      DBG_INFO(0, "Loading message engine");
      mngin=_hbci->engineFromConfig(bank.ref().country(),
                                    bank.ref().bankCode(),
                                    bank.ref().hbciVersion(),
				    cfg1);
      if (!err.isOk())
	return err;
    }
    catch (Error xerr) {
      return Error("Loader::loadBank", xerr);
    }
  } // if group exists
  else {
    // setup the default engine
    try {
      mngin=_hbci->engineFactory(bank.ref().country(),
				 bank.ref().bankCode(),
				 bank.ref().hbciVersion(),
				 LOADER_DEFAULT_ENGINE);
    }
    catch (Error xerr) {
      return Error("Loader::loadBank", xerr);
    }
  }
  bank.ref().setMessageEngine(mngin);
  DBG_INFO(0, "Message engine ready.");

  // load transporter
  cfg1=GWEN_DB_GetGroup(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "transport");
  if (cfg1) {
    try {
      DBG_INFO(0, "Loading transporter");
      xport=_hbci->transporterFromConfig(bank.ref().address(), "3000", cfg1);
      if (!err.isOk())
	return err;
    }
    catch (Error xerr) {
      return Error("Loader::loadBank", xerr);
    }
  } // if group exists
  else {
    // setup the default engine
    try {
      xport=_hbci->transporterFactory(bank.ref().address(),
				      "3000",
				      LOADER_DEFAULT_TRANSPORTER_TIMEOUT,
				      LOADER_DEFAULT_TRANSPORTER);
    }
    catch (Error xerr) {
      return Error("Loader::loadBank", xerr);
    }
  }
  bank.ref().setTransporter(xport);
  DBG_INFO(0, "Transporter ready.");

  // load all users
  cfg1=GWEN_DB_GetGroup(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "user");
  if (cfg1) {
    cfg1=GWEN_DB_GetFirstGroup(cfg1);
    while(cfg1) {
      DBG_INFO(0, "Loading user \"%s\"", GWEN_DB_GroupName(cfg1));
      err=loadUser(bank, cfg1);
      if (!err.isOk())
	return err;
      // add user to list
      cfg1=GWEN_DB_GetNextGroup(cfg1);
    } // while there are users
  } // if group exists

  // add bank
  _hbci->addBank(bank);

  return Error();

}


Error Loader::loadAll(GWEN_DB_NODE *cfg,
		      unsigned int flags) {
  GWEN_DB_NODE *cfg1;
  Error err;


  Outbox::setNextId(GWEN_DB_GetIntValue(cfg, "nextjobid",0, 1));

  // load all banks
  cfg1=GWEN_DB_GetGroup(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                        "bank");
  if (cfg1) {
    cfg1=GWEN_DB_GetFirstGroup(cfg1);
    while(cfg1) {
      DBG_INFO(0, "Loading bank \"%s\"", GWEN_DB_GroupName(cfg1));
      err=loadBank(cfg1,flags);
      if (!err.isOk())
	return err;
      cfg1=GWEN_DB_GetNextGroup(cfg1);
    } // while there are banks
  } // if group exists
  else {
    DBG_WARN(0, "No bank!");
  }
  return Error();
}


} // namespace HBCI
