/***************************************************************************
 $RCSfile: loader.h,v $
                             -------------------
    cvs         : $Id: loader.h,v 1.3 2004/01/19 20:20:34 aquamaniac Exp $
    begin       : Sat Jun 08 2002
    copyright   : (C) 2001 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef HBCILOADER_H
#define HBCILOADER_H

#include <string>
#include <list>
#include <openhbci2/dllimport.h>
#include <gwenhywfar/db.h>


#define LOADER_DEFAULT_ENGINE "msgxml"
#define LOADER_DEFAULT_TRANSPORTER "xptcp"
#define LOADER_DEFAULT_TRANSPORTER_TIMEOUT 500

namespace HBCI {
  class Loader;
}
/**
 * Flags for loading and saving
 */
#define HBCILOADER_WITH_TRANSACTIONS    0x0001
#define HBCILOADER_WITH_MESSAGES        0x0002
#define HBCILOADER_WITH_STANDINGORDERS  0x0004
#define HBCILOADER_DEFAULT (\
  HBCILOADER_WITH_TRANSACTIONS| \
  HBCILOADER_WITH_MESSAGES | \
  HBCILOADER_WITH_STANDINGORDERS \
  )

#include <openhbci2/api.h>

namespace HBCI {

/**
 * This class saves and loads all OpenHCBI data and is called from HBCIAPI.
 * @short Saves and loads all OpenHCBI data and is called from HBCIAPI.
 * @author Martin Preuss<martin@libchipcard.de>
 */
class DLLIMPORT Loader{
  API *_hbci;

private:

public:
  Loader(API *hbci);
  ~Loader();

  Error saveCustomer(Pointer<Customer> c,
		     GWEN_DB_NODE *cfg);
  Error saveUser(Pointer<User> c,
		 GWEN_DB_NODE *cfg);
  Error saveBank(Pointer<Bank> b,
		 GWEN_DB_NODE *cfg,
		 unsigned int flags=HBCILOADER_DEFAULT);

  Error saveAll(GWEN_DB_NODE *cfg,
		unsigned int flags=HBCILOADER_DEFAULT);

  Error loadCustomer(Pointer<User> u,
		     Pointer<Customer> &cu,
		     GWEN_DB_NODE *cfg);
  Error loadUser(Pointer<Bank> b,
		 GWEN_DB_NODE *cfg);
  Error loadBank(GWEN_DB_NODE *cfg,
		 unsigned int flags=HBCILOADER_DEFAULT);

  Error loadAll(GWEN_DB_NODE *cfg,
		unsigned int flags=HBCILOADER_DEFAULT);

};


} /* namespace HBCI */


#endif
