/***************************************************************************
  $RCSfile: transporter.h,v $
  -------------------
  cvs         : $Id: transporter.h,v 1.4 2004/01/29 11:52:13 cstim Exp $
  begin       : Sat Jul 05 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TRANSPORTER_H
#define TRANSPORTER_H

#ifdef __cplusplus
#include <string>
using namespace std;
#endif /* __cplusplus */

#include <openhbci2/error.h>

typedef enum {
  TransportTypeBTX=1,
  TransportTypeTCP,
  TransportTypeSSL
} TransportType;

#ifdef __cplusplus

namespace HBCI {

  /**
   * This is an abstract base class for the transport layer.
   * It is not part of the OpenHBCI API (it is part of the OpenHBCI
   * plugin interface, though)
   * @author Martin Preuss<martin@libchipcard.de>
   */
  class Transporter {
  private:
    string _address;
    string _port;
    int _timeout;

  public:
    Transporter(const string &address,
		const string &port,
		int timeout);
    virtual ~Transporter();
    virtual const string &address() const { return _address;};
    virtual const string &port() const { return _port;};
    virtual TransportType transportType() const =0;

    virtual Error startOpen()=0;
    virtual Error checkOpen()=0;
    virtual bool isOpen()=0;
    virtual Error close()=0;
    virtual string readBytes(int maxb)=0;
    virtual Error writeBytes(const string &buffer)=0;
    virtual string transporterTypeName() const =0;
    virtual int timeout() const { return _timeout;};
  };


}; /* namespace */

typedef HBCI::Transporter HBCI_Transporter;
extern "C" {
#else /* __cplusplus */
  typedef struct HBCI_Transporter HBCI_Transporter;
#endif /* #ifdef __cplusplus */

  const char *HBCI_Transporter_address(const HBCI_Transporter *t);
  const char *HBCI_Transporter_port(const HBCI_Transporter *t);
  TransportType HBCI_Transporter_transportType(const HBCI_Transporter *t);

#ifdef __cplusplus
}
#endif /* #ifdef __cplusplus */


#endif

