

#define GWEN_EXTEND_WAITCALLBACK

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>

#include "callback_p.h"


GWEN_INHERIT(GWEN_WAITCALLBACK, HBCI_CALLBACK);


GWEN_WAITCALLBACK *HBCI_Callback_new(const char *id,
				     HBCI_Hbci *hbci){
  GWEN_WAITCALLBACK *ctx;
  HBCI_CALLBACK *cb;

  ctx=GWEN_WaitCallback_new(id);
  GWEN_NEW_OBJECT(HBCI_CALLBACK, cb);
  GWEN_INHERIT_SETDATA(GWEN_WAITCALLBACK, HBCI_CALLBACK,
                       ctx, cb,
		       HBCI_Callback_free);
  cb->hbci=hbci;
  GWEN_WaitCallback_SetCheckAbortFn(ctx, HBCI_Callback_CheckAbort);
  GWEN_WaitCallback_SetInstantiateFn(ctx, HBCI_Callback_Instantiate);
  GWEN_WaitCallback_SetLogFn(ctx, HBCI_Callback_Log);
  return ctx;
}



void HBCI_Callback_free(void *bp, void *p){
  GWEN_WAITCALLBACK *ctx;
  HBCI_CALLBACK *cb;

  ctx=(GWEN_WAITCALLBACK*)bp;
  cb=(HBCI_CALLBACK*)p;

  free(cb);
}



GWEN_WAITCALLBACK *HBCI_Callback_Instantiate(GWEN_WAITCALLBACK *ctx){
  GWEN_WAITCALLBACK *nctx;
  HBCI_CALLBACK *cb;

  assert(ctx);
  cb=GWEN_INHERIT_GETDATA(GWEN_WAITCALLBACK, HBCI_CALLBACK, ctx);
  nctx=HBCI_Callback_new(GWEN_WaitCallback_GetId(ctx), cb->hbci);

  DBG_INFO(0, "Instantiating callback \"%s\"",
           GWEN_WaitCallback_GetId(ctx));
  GWEN_WaitCallback_SetDistance(nctx, GWEN_WaitCallback_GetDistance(ctx));
  return nctx;
}



void HBCI_Callback_Log(GWEN_WAITCALLBACK *ctx,
                       unsigned int level,
                       unsigned int logLevel,
                       const char *s){
  HBCI_CALLBACK *cb;
  HBCI_Interactor *ia;

  cb=GWEN_INHERIT_GETDATA(GWEN_WAITCALLBACK, HBCI_CALLBACK, ctx);
  assert(cb);

  ia=HBCI_Hbci_interactor(cb->hbci);
  assert(ia);
  DBG_NOTICE(0, "Callback log: %s", s);
}



GWEN_WAITCALLBACK_RESULT
HBCI_Callback_CheckAbort(GWEN_WAITCALLBACK *ctx,
                         unsigned int level){
  HBCI_CALLBACK *cb;
  HBCI_Interactor *ia;

  cb=GWEN_INHERIT_GETDATA(GWEN_WAITCALLBACK, HBCI_CALLBACK, ctx);
  assert(cb);

  ia=HBCI_Hbci_interactor(cb->hbci);
  assert(ia);

  if (HBCI_Interactor_keepAlive(ia)) {
    return GWEN_WaitCallbackResult_Continue;
  }
  return GWEN_WaitCallbackResult_Abort;
}




