/***************************************************************************
 $RCSfile: msgengine.h,v $
                             -------------------
    cvs         : $Id: msgengine.h,v 1.6 2004/02/07 11:23:02 cstim Exp $
    begin       : Sun Jul 06 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef MSGENGINE_H
#define MSGENGINE_H

#ifdef __cplusplus
namespace HBCI {
  class MessageEngine;
};
typedef HBCI::MessageEngine HBCI_MessageEngine;
#else /* __cplusplus */
typedef struct HBCI_MessageEngine HBCI_MessageEngine;
#endif /* __cplusplus */


#include <openhbci2/msgqueue.h>
#include <openhbci2/hbci.h>
#include <openhbci2/pointer.h>
#include <openhbci2/customer.h>
#include <openhbci2/transporter.h>


#ifdef __cplusplus

namespace HBCI {

  /**
   * @internal
   * This is an abstract base class for message engines.
   * It is not part of the OpenHBCI API (it is part of the OpenHBCI
   * plugin interface, though)
   * @author Martin Preuss<martin@libchipcard.de>
   */
  class MessageEngine {
  private:
    Hbci *_hbci;
    int _country;
    string _bankCode;

  public:
    MessageEngine(Hbci *hbci, int country, const string &bankCode);
    virtual ~MessageEngine();

    Hbci *hbci() const { return _hbci;};
    int country() const { return _country;};
    const string &bankCode() const { return _bankCode;};

    virtual void setHbciVersion(int v) =0;

    /**
     * @internal
     * Call this before executing queues with this engine.
     * After executing the last queue you should call @ref endSession to
     * free ressources.
     */
    virtual Error startSession(Pointer<Customer> owner,
                               Pointer<Transporter> tx)=0;

    /**
     * @internal
     * Call this after executing queues with this engine.
     */
    virtual Error endSession()=0;
    virtual Error executeQueue(Pointer<MsgQueue> mq)=0;

    /**
     * Creates a MsgJob object. This way OpenHBCI is fully independant of
     * the actual implementation of the MsgJob.
     */
    virtual Pointer<MsgJob> jobFactory(const string &name,
				       Pointer<Customer> cu,
				       const string &account="") =0;

    /**
     * Creates MsgJob object. This way OpenHBCI is fully independant of
     * the actual implementation of the MsgJob.
     */
    virtual Pointer<MsgQueue> queueFactory(Pointer<Customer> cu) =0;

    virtual string engineType() const =0;

  };
};

extern "C" {
#endif /* __cplusplus */

  HBCI_Hbci *HBCI_MessageEngine_hbci(const HBCI_MessageEngine *h);
  int HBCI_MessageEngine_country(const HBCI_MessageEngine *h);
  const char *HBCI_MessageEngine_bankCode(const HBCI_MessageEngine *h);

#ifdef __cplusplus
}
#endif /* #ifdef __cplusplus */
#endif




