/***************************************************************************
 $RCSfile: msgjob.cpp,v $
                             -------------------
    cvs         : $Id: msgjob.cpp,v 1.3 2004/01/19 18:53:50 aquamaniac Exp $
    begin       : Sun Jul 06 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "msgjob.h"
#include "hbcistring.h"
#include <assert.h>


namespace HBCI {


MessageReference::MessageReference()
:_msgNumber(-1)
{
}


MessageReference::MessageReference(const string &deg)
:_msgNumber(-1)
{
  unsigned int spos;

  spos=0;
  _dialogId=String::nextDEG(deg,spos);
  spos+=String::nextDEG(deg,spos).length()+1;
  _msgNumber=atoi(String::nextDEG(deg,spos).c_str());
}


MessageReference::MessageReference(const string &did, int msgnum)
:_dialogId(did)
,_msgNumber(msgnum)
{
}


MessageReference::~MessageReference(){
}


string MessageReference::toString() const {
  string result;

  result=_dialogId+":";
  result+=String::num2string(_msgNumber);
  return result;
}


bool MessageReference::operator==(const MessageReference &ref) const{
  return (ref._dialogId==_dialogId &&
          ref._msgNumber==_msgNumber);
}
int MessageReference::cmp(const MessageReference &other) const
{
  return (operator==(other) ? 0 :
	  (_dialogId < other._dialogId ? -1 :
	   (_dialogId > other._dialogId ? 1 :
	    (_msgNumber < other._msgNumber ? -1 : 1))));
}



HBCI_MessageReference *
HBCI_MessageReference_new(const char *did, int msgnum)
{
  return new HBCI_MessageReference(did ? did : "", msgnum);
}
HBCI_MessageReference *
HBCI_MessageReference_new_deg(const char *deg)
{
  return new HBCI::MessageReference(deg ? deg : "");
}
void HBCI_MessageReference_delete(HBCI_MessageReference *h)
{
  delete (h);
}
const char *
HBCI_MessageReference_dialogId(const HBCI_MessageReference *h)
{
  assert(h);
  return h->dialogId().c_str();
}
int 
HBCI_MessageReference_messageNumber(const HBCI_MessageReference *h)
{
  assert(h);
  return h->messageNumber();
}
char * 
HBCI_MessageReference_toString(const HBCI_MessageReference *h)
{
  assert(h);
  return hbci_strdup(h->toString());
}
int HBCI_MessageReference_equal(const HBCI_MessageReference *h1,
				const HBCI_MessageReference *h2)
{
  assert(h1);
  assert(h2);
  return h1->operator==(*h2);
}
int HBCI_MessageReference_cmp(const HBCI_MessageReference *h1,
			      const HBCI_MessageReference *h2)
{
  assert(h1);
  assert(h2);
  return h1->cmp(*h2);
}







MsgJob::Result::Result(int code, const string &message,
                       const string &param,
		       const string &property,
                       const string &value,
                       const string &explanation)
:_code(code)
,_message(message)
,_param(param)
,_property(property)
,_value(value)
,_explanation(explanation)
{
}


MsgJob::Result::~Result() {
}


MsgJob::MsgJob(Pointer<Customer> owner,
	       const string &name, const string &accountnr)
:_name(name), _accountnr(accountnr), _owner(owner)
{
}


MsgJob::~MsgJob(){
}




}; // namespace



int HBCI_MsgJob_Result_resultCode(const HBCI_MsgJob_Result *r)
{
    assert(r);
    return r->resultCode();
}
const char *HBCI_MsgJob_Result_resultMessage(const HBCI_MsgJob_Result *r)
{
    assert(r);
    return r->resultMessage().c_str();
}
const char *HBCI_MsgJob_Result_param(const HBCI_MsgJob_Result *r)
{
    assert(r);
    return r->param().c_str();
}
const char *HBCI_MsgJob_Result_property(const HBCI_MsgJob_Result *r)
{
    assert(r);
    return r->property().c_str();
}
const char *HBCI_MsgJob_Result_value(const HBCI_MsgJob_Result *r)
{
    assert(r);
    return r->value().c_str();
}
const char *HBCI_MsgJob_Result_explanation(const HBCI_MsgJob_Result *r)
{
    assert(r);
    return r->explanation().c_str();
}


int list_HBCI_MsgJob_Result_size(const list_HBCI_MsgJob_Result *l)
{
    assert(l);
    return l->size();
}
void *
list_HBCI_MsgJob_Result_foreach (const list_HBCI_MsgJob_Result *list,
				 list_HBCI_MsgJob_Result_cb func,
				 void *user_data)
{
    assert(list);
    assert(func);
    std::list<HBCI::MsgJob::Result>::const_iterator iter, end;
    void *ret = 0l;

    end = list->end();
    for ( iter = list->begin(); iter != end; iter++ ) {
	ret = (*func)( &(*iter), user_data);
	if (ret)
	    break;
    }
    return ret;
}
