/***************************************************************************
 $RCSfile: msgqueue.cpp,v $
                             -------------------
    cvs         : $Id: msgqueue.cpp,v 1.3 2003/11/22 20:09:00 aquamaniac Exp $
    begin       : Sun Jul 06 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "msgqueue.h"
#include <gwenhywfar/stringlist.h>
#include <gwenhywfar/debug.h>


namespace HBCI {

MsgQueue::Result::Result(int code, const string &message,
                         const string &jobname,
                         const string &explanation)
    :_code(code)
    ,_message(message)
    ,_jobname(jobname)
    ,_explanation(explanation)
{
}


MsgQueue::Result::~Result() {
}


MsgQueue::MsgQueue(){
}


MsgQueue::~MsgQueue(){
}


void MsgQueue::addResult(const Result &r) {
  _results.push_back(r);
}


Error MsgQueue::addJob(Pointer<MsgJob> j){
  _jobs.push_back(j);
  return Error();
}



HBCI_QUEUECHECK_RESULT MsgQueue::checkAddJob(Pointer<MsgJob> j){
  GWEN_STRINGLIST *sl;
  list<Pointer<MsgJob> >::iterator it;
  int jobsOfThisKind;
  int diffjobs;

  if (j.ref().single() && !_jobs.empty()) {
    DBG_INFO(0,
             "Job \"%s\" wants to be alone but queue's not empty",
             j.ref().name().c_str());
    return HBCI_QUEUECHECK_RESULT_JOB;
  }
  sl=GWEN_StringList_new();
  GWEN_StringList_AppendString(sl, j.ref().name().c_str(), 0, 1);
  jobsOfThisKind=1;
  for (it=_jobs.begin();
       it!=_jobs.end();
       it++) {
    if ((*it).ref().single()) {
      DBG_INFO(0, "Jobs \"%s\" wants to be alone",
               (*it).ref().name().c_str());
      GWEN_StringList_free(sl);
      return HBCI_QUEUECHECK_RESULT_QUEUE_FULL;
    }
    GWEN_StringList_AppendString(sl, (*it).ref().name().c_str(), 0, 1);
    if ((*it).ref().name()==j.ref().name())
      jobsOfThisKind++;
  } // for

  diffjobs=GWEN_StringList_Count(sl);
  GWEN_StringList_free(sl);
  if (diffjobs>maxDifferentJobsPerQueue() && maxDifferentJobsPerQueue()){
    DBG_INFO(0, "Too many different jobs (%d)", diffjobs);
    return HBCI_QUEUECHECK_RESULT_JOB;
  }
  if (jobsOfThisKind>j.ref().jobsPerMessage() && j.ref().jobsPerMessage()){
    DBG_INFO(0, "Too many jobs of \"%s\" (%d)",
             j.ref().name().c_str(),
             jobsOfThisKind);
    return HBCI_QUEUECHECK_RESULT_JOB;
  }
  DBG_INFO(0,"Ok to add job \"%s\"", j.ref().name().c_str());
  return HBCI_QUEUECHECK_RESULT_OK;
}




}; // namespace */




