/***************************************************************************
 $RCSfile: msgqueue.h,v $
                             -------------------
    cvs         : $Id: msgqueue.h,v 1.3 2004/01/19 20:20:35 aquamaniac Exp $
    begin       : Sun Jul 06 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef MSGQUEUE_H
#define MSGQUEUE_H

#ifdef __cplusplus

namespace HBCI {
  class MsgQueue;
}


#include <openhbci2/msgjob.h>
#include <openhbci2/pointer.h>
#include <openhbci2/customer.h>

#include <list>
using namespace std;


typedef enum {
  HBCI_QUEUECHECK_RESULT_OK=0,
  HBCI_QUEUECHECK_RESULT_ERROR,
  /** Job does not fit */
  HBCI_QUEUECHECK_RESULT_JOB,
  /** Queue is full */
  HBCI_QUEUECHECK_RESULT_QUEUE_FULL
} HBCI_QUEUECHECK_RESULT;


namespace HBCI {

  /**
   * @internal
   * This is an abstract base class for message queues.
   * It is not part of the OpenHBCI API (it is part of the OpenHBCI
   * plugin interface, though)
   * You are probably looking for @ref Outbox.
   * @author Martin Preuss<martin@libchipcard.de>
   */
  class MsgQueue {
  public:
    class DLLIMPORT Result {
    private:
      int _code;
      string _message;
      string _jobname;
      string _explanation;
    public:
      Result(int code=-1,
	     const string &message="",
             const string &jobname="",
             const string &explanation="");
      ~Result();
      int resultCode() const { return _code; };
      string resultMessage() const { return _message;};
      const string &jobname() const { return _jobname;};
      /**
       * Returns an explanatory string for this result.
       * Not all MessageEngines have to support this feature.
       */
      const string &explanation() const { return _explanation;};
    };

  private:
    Pointer<Customer> _customer;
    list<Result> _results;

  protected:
    list<Pointer<MsgJob> > _jobs;

  public:
    MsgQueue();
    virtual ~MsgQueue();
    virtual HBCI_QUEUECHECK_RESULT checkAddJob(Pointer<MsgJob> j);
    virtual Error addJob(Pointer<MsgJob> j);
    virtual const list<Pointer<MsgJob> > &jobs() { return _jobs;};
    virtual const list<Result> &results() const { return _results;};
    virtual bool empty() { return _jobs.empty(); };

    void addResult(const Result &r);

    virtual int maxDifferentJobsPerQueue() { return 1;};

    /** @name Properties
     *
     * These are properties which do not belong to a particular job but
     * rather to the whole message (like updated UPDs/BPDs).
     */
    /*@{*/
    virtual string getProperty(const string &name,
			       const string &defValue)=0;
    virtual int getIntProperty(const string &name,
			       int defValue)=0;
    virtual string getBinProperty(const string &name,
				  const string &defValue)=0;

    /**
     * Returns a private copy of the response stored within this queue.
     * The caller is responsible for freeing this data by calling
     * Config_free()
     */
    virtual GWEN_DB_NODE *queueResponse() =0;

    /*@}*/
  };


} ; /* namespace */


#endif /* #ifdef __cplusplus */

#endif



