/***************************************************************************
  $RCSfile: msgengineplugin.h,v $
  -------------------
  cvs         : $Id: msgengineplugin.h,v 1.4 2004/01/19 20:20:34 aquamaniac Exp $
  begin       : Mon Apr 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/
/**
 * @file msgengineplugin.h Declaration of the abstract base class for a
 * message engine plugin.
 */

#ifndef MSGENGINEPLUGIN_H
#define MSGENGINEPLUGIN_H

#include <openhbci2/pointer.h>
#include <openhbci2/plugin.h>
#include <openhbci2/msgengine.h>
#include <gwenhywfar/db.h>

#ifdef __cplusplus
#include <string>
using namespace std;

namespace HBCI {

/**
 * @brief This is the abstract base class for a message engine plugin.
 *
 * Note: If you are writing a new engine plugin, then the *first*
 * class your new engine class has to inherit is "HBCI::MessageEngine" (not
 * the other way round)!
 *
 * @author Martin Preuss<martin@libchipcard.de>
 * @see @ref pluginpage
 */
class MessageEnginePlugin: public Plugin {
private:
public:
  /** Default Constructor */
  MessageEnginePlugin(Hbci *hbci);
  /** Default Destructor */
  virtual ~MessageEnginePlugin();

  /**
   * Returns the type of this plugin.
   */
  virtual PluginType pluginType() const { return PluginTypeMessage;};

  /**
   * This method is used to create a medium object using the data given in
   * a configuration file.
   * There are some mandatory variables in this group:
   * <ul>
   *  <li>
   *   mediumtype: This is used to lookup the medium plugin that is
   *   responsible for this medium
   *  </li>
   *  <li>
   *   mediumname: This is used to identify a medium. Each medium MUST have
   *   a unique name. RDHFiles use the path and name of the keyfile here,
   *   while chipcard media use the card's serial number.
   *  </li>
   * </ul>
   * The plugin may additionally use whichever variables it finds necessary.
   */
  virtual Pointer<MessageEngine> engineFromConfig(int country,
						  const string &bankCode,
                                                  int hversion,
                                                  GWEN_DB_NODE *cfg) =0;

  /**
   * This method is used to store information from a medium to a configuration
   * file (in order to be restorable by @ref mediumFromConfig).
   * See @ref mediumFromConfig for details about the config group.
   */
  virtual Error engineToConfig(Pointer<MessageEngine> m,
                               GWEN_DB_NODE  *cfg) =0;

  /**
   */
  virtual Pointer<MessageEngine> engineFactory(int country,
					       const string &bankCode,
					       int hversion) =0;

};


}; /* namespace */
#endif /* __cplusplus */

#endif

