/***************************************************************************
  $RCSfile: pluginmanager.h,v $
  -------------------
  cvs         : $Id: pluginmanager.h,v 1.2 2004/01/19 20:20:34 aquamaniac Exp $
  begin       : Sat Jul 05 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PLUGINMANAGER_H
#define PLUGINMANAGER_H

#ifdef __cplusplus

namespace HBCI {
  class Hbci;
  class PluginManager;
};


typedef HBCI::PluginManager PluginManager;

#include <openhbci2/pointer.h>
#include <openhbci2/pluginfile.h>
#include <list>


namespace HBCI {

  /**
   * This class handles plugin loading.
   * @author Martin Preuss<martin@libchipcard.de>
   */
  class PluginManager {
  private:
    Hbci *_hbci;
    list<Pointer<PluginFile> > _files;
    list<Pointer<Plugin> > _plugins;

    string _getLibraryPrefix(const string &path);
    string _getTypeSubdir(PluginType t);
    Error _loadPlugin(const string &path, PluginType t);
    Error _tryToLoadPlugin(const list<string> &dnames,
			   PluginType pt,
			   const string &mname,
			   unsigned int wantsoversion,
			   unsigned int wantsoage);

  public:
    PluginManager(Hbci *hbci);
    ~PluginManager();

    /**
     * This method registers a given plugin. If there already is a plugin of
     * the given type with the given name then the given plugin will not be
     * registered.
     */
    Error registerPlugin(Pointer<Plugin> p);

    /**
     * Checks for a plugin. This does not load any plugin !
     */
    Pointer<Plugin> findPlugin(const string &name,
			       PluginType pt);

    /**
     * This method tries to make sure that the given plugin is available.
     * It tries to load the plugin if it not already is loaded.
     */
    Pointer<Plugin> ensurePlugin(const string &mname,
				 PluginType pt) throw (Error);

    /**
     * This method only <b>loads</b> a plugin, it does not register it
     * nor does it add the PluginFile to the internal list !
     * When deleting these objects please bear in mind that a Plugin must
     * be deleted <b>before</b> before its pluginfile is deleted !
     */
    Error loadPlugin(const string &path,
		     PluginType t,
		     Pointer<PluginFile> &pf,
		     Pointer<Plugin> &pl);

    const list<Pointer<Plugin> > &plugins() const { return _plugins;};

  };

} /* namepsace */
#endif /* __cplusplus */


#endif /* PLUGINMANAGER_H */

