/***************************************************************************
 $RCSfile: transaction.cpp,v $
                             -------------------
    cvs         : $Id: transaction.cpp,v 1.6 2004/01/21 16:46:43 cstim Exp $
    begin       : Sat Jun 08 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 Changes

 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "transaction.h"
#include "hbcistring.h"
#include <assert.h>
#include "error.h"


namespace HBCI {

  
  Transaction::Transaction(GWEN_DB_NODE *n)
    : _n(n)
  {
  }
  
  
  Transaction::~Transaction() {
  }

  const Date& Transaction::valutaDate() const 
    {
      if (get("valutaDate") != tmp_valutadate.toString())
	tmp_valutadate = Date(get("valutaDate"));
      return tmp_valutadate;
    }
  const Date& Transaction::date() const
    {
      if (get("date") != tmp_date.toString())
	tmp_date = Date(get("date"));
      return tmp_date;
    }
  const Value& Transaction::value() const 
    {
      if (tmp_value.toString(true) != value_value() ||
	  tmp_value.getCurrency() != value_currency())
	tmp_value = Value(value_value(), value_currency());
      return tmp_value;
    }
  void Transaction::setValue(const Value& v)
  {
      setValue_value(v.toString(true).c_str());
      setValue_currency(v.getCurrency().c_str());
  }
  const Value &Transaction::originalValue() const
    {
      if (tmp_origValue.toString(true) != originalValue_value() ||
	  tmp_origValue.getCurrency() != originalValue_currency())
	tmp_origValue = Value(originalValue_value(), originalValue_currency());
      return tmp_origValue;
    }
  const Value &Transaction::charge() const
    {
      if (tmp_charges.toString(true) != charge_value() ||
	  tmp_charges.getCurrency() != charge_currency())
	tmp_charges = Value(charge_value(), charge_currency());
      return tmp_charges;
    }

  namespace {
    // Both functions below do the same thing, so put it into one
    // common function.
    void gwen_to_list_with_cache(GWEN_DB_NODE *node,
				 const char *variablename,
				 list<string> &cachelist)
    {
      const char *p;
      list<string> result;
      bool cache_valid = !cachelist.empty();
      unsigned ROWS=20;
      // Iterator necessary because in a list we can't access elements
      // by index
      list<string>::const_iterator cache_iter = cachelist.begin();
    
      for (unsigned i=0; i<ROWS; i++) {
	p=GWEN_DB_GetCharValue(node, variablename, i, 0);
	if (!p)
	  break;
	if (cache_valid && (i >= cachelist.size() || p != *(cache_iter++)))
	  cache_valid = false;
	result.push_back(p);
      } // for

      if (cache_valid == false)
	// Copy all elements
	cachelist = result;
    }
  }

  const list<string>& Transaction::description() const
  {
    gwen_to_list_with_cache(_n, "purpose", tmp_purpose);
    return tmp_purpose;
  }

  const list<string>& Transaction::otherName() const
  {
    gwen_to_list_with_cache(_n, "othername", tmp_othername);
    return tmp_othername;
  }
  
} // namespace HBCI

HBCI_Transaction * HBCI_Transaction_new(GWEN_DB_NODE *n)
{
    return new HBCI_Transaction(n);
}
void HBCI_Transaction_delete(HBCI_Transaction *t)
{
    delete t;
}
GWEN_DB_NODE *HBCI_Transaction_node(HBCI_Transaction *t)
{
    assert(t);
    return t->node();
}


/*int HBCI_Transaction_ourCountryCode(const HBCI_Transaction *t)
{
    assert(t);
    return t->ourCountryCode();
}
void 
HBCI_Transaction_setOurCountryCode(HBCI_Transaction *t, int i)
{
    assert(t);
    t->setOurCountryCode(i);
}*/
const char * HBCI_Transaction_ourBankCode(const HBCI_Transaction *t)
{
    assert(t);
    return t->ourBankCode();
}
void 
HBCI_Transaction_setOurBankCode(HBCI_Transaction *t, const char *code)
{
    assert(t);
    t->setOurBankCode((code ? code : ""));
}
const char * HBCI_Transaction_ourAccountId(const HBCI_Transaction *t)
{
    assert(t);
    return t->ourAccountId();
}
void 
HBCI_Transaction_setOurAccountId(HBCI_Transaction *t, const char *id)
{
    assert(t);
    t->setOurAccountId((id ? id : ""));
}
/*const char * HBCI_Transaction_ourSuffix(const HBCI_Transaction *t)
{
    assert(t);
    return t->ourSuffix();
}
void 
HBCI_Transaction_setOurSuffix(HBCI_Transaction *t, const char *id)
{
    assert(t);
    t->setOurSuffix((id ? id : ""));
}*/
const HBCI_Date* HBCI_Transaction_valutaDate(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->valutaDate());
}
void 
HBCI_Transaction_setValutaDate(HBCI_Transaction *t, const HBCI_Date *d)
{
    assert(t);
    assert(d);
    t->setValutaDate(*d);
}
const HBCI_Date* HBCI_Transaction_date(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->date());
}
void 
HBCI_Transaction_setDate(HBCI_Transaction *t, const HBCI_Date *d)
{
    assert(t);
    assert(d);
    t->setDate(*d);
}
const HBCI_Value* HBCI_Transaction_value(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->value());
}
const HBCI_Value *HBCI_Transaction_originalValue(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->originalValue());
}
const HBCI_Value *HBCI_Transaction_charge(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->charge());
}
void 
HBCI_Transaction_setValue(HBCI_Transaction *t, const HBCI_Value *v)
{
    assert(t);
    assert(v);
    t->setValue(*v);
}
/*const char * HBCI_Transaction_transactionKey(const HBCI_Transaction *t)
{
    assert(t);
    return t->transactionKey();
}
void 
HBCI_Transaction_setTransactionKey(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setTransactionKey((s ? s : ""));
}*/
const char * HBCI_Transaction_customerReference(const HBCI_Transaction *t)
{
    assert(t);
    return t->customerReference();
}
void 
HBCI_Transaction_setCustomerReference(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setCustomerReference((s ? s : ""));
}
const char * HBCI_Transaction_bankReference(const HBCI_Transaction *t)
{
    assert(t);
    return t->bankReference();
}
void 
HBCI_Transaction_setBankReference(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setBankReference((s ? s : ""));
}


// Now follows the obnoxious "Mehrzweckfeld :86:"

int HBCI_Transaction_transactionCode(const HBCI_Transaction *t)
{
    assert(t);
    return t->transactionCode();
}
void 
HBCI_Transaction_setTransactionCode(HBCI_Transaction *t, int c)
{
    assert(t);
    t->setTransactionCode(c);
}
/*const char * HBCI_Transaction_transactionText(const HBCI_Transaction *t)
{
    assert(t);
    return t->transactionText();
}
void 
HBCI_Transaction_setTransactionText(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setTransactionText((s ? s : ""));
}*/
/*const char * HBCI_Transaction_primanota(const HBCI_Transaction *t)
{
    assert(t);
    return t->primanota();
}
void 
HBCI_Transaction_setPrimanota(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setPrimanota((s ? s : ""));
}*/
const list_string* HBCI_Transaction_description(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->description());
}
void 
HBCI_Transaction_addDescription(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->addDescription((s ? s : ""));
}
/*int HBCI_Transaction_otherCountryCode(const HBCI_Transaction *t)
{
    assert(t);
    return t->otherCountryCode();
}
void 
HBCI_Transaction_setOtherCountryCode(HBCI_Transaction *t, int i)
{
    assert(t);
    t->setOtherCountryCode(i);
}*/
const char * HBCI_Transaction_otherBankCode(const HBCI_Transaction *t)
{
    assert(t);
    return t->otherBankCode();
}
void 
HBCI_Transaction_setOtherBankCode(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setOtherBankCode((s ? s : ""));
}
const char * HBCI_Transaction_otherAccountId(const HBCI_Transaction *t)
{
    assert(t);
    return t->otherAccountId();
}
void 
HBCI_Transaction_setOtherAccountId(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->setOtherAccountId((s ? s : ""));
}
/*const char * HBCI_Transaction_otherSuffix(const HBCI_Transaction *t)
{
    assert(t);
    return t->otherSuffix();
}
void 
HBCI_Transaction_setOtherSuffix(HBCI_Transaction *t, const char *id)
{
    assert(t);
    t->setOtherSuffix((id ? id : ""));
}*/
const list_string* HBCI_Transaction_otherName(const HBCI_Transaction *t)
{
    assert(t);
    return &(t->otherName());
}
void 
HBCI_Transaction_addOtherName(HBCI_Transaction *t, const char *s)
{
    assert(t);
    t->addOtherName((s ? s : ""));
}



struct _list_HBCI_Transaction 
{
    GWEN_DB_NODE *n;
};

/*list_HBCI_Transaction *list_HBCI_Transaction_new()
{
    list_HBCI_Transaction *res = new list_HBCI_Transaction();
    res->n = GWEN_DB_Group_new("transfers");
    return res;
    }*/

// void list_HBCI_Transaction_push_back(list_HBCI_Transaction *l, 
// 				     const HBCI_Transaction *trans)
// {
//     assert(l);
//     assert(trans);
//     /* FIXME: Is this the correct way to set up a "list of
//      * transactions" so that the resulting GWEN_DB_NODE can directly
//      * be passed to the JobMultiTransfer and JobMultiDebitNode? */
//     GWEN_DB_AddGroup(l->n, 
// 		     GWEN_DB_Group_dup(const_cast<HBCI_Transaction*>(trans)->node()));
// }

/*GWEN_DB_NODE *list_HBCI_Transaction_node(list_HBCI_Transaction *l)
{
    assert(l);
    return l->n;
    }*/

/*void list_HBCI_Transaction_delete(list_HBCI_Transaction *l)
{
    GWEN_DB_Group_free(l->n);
    delete l;
    }*/
/*int list_HBCI_Transaction_size(const list_HBCI_Transaction *l)
{
    assert(l);
    int i;
    GWEN_DB_NODE *p;
    for (i=0; i<100; i++) {
	p=GWEN_DB_GetGroup(l->n, i, "transfer");
	if (!p)
	    break;
    } // for

    return i;
    }*/
/*void *list_HBCI_Transaction_foreach (const list_HBCI_Transaction *list, 
				     list_HBCI_Transaction_cb func,
				     void *user_data)
{
    assert(list);
    assert(func);

    void *ret = 0l;

    int i;
    GWEN_DB_NODE *p;
    for (i=0; i<100; i++) {
	p=GWEN_DB_GetGroup(list->n, i, "transfer");
	if (!p)
	    break;
	HBCI_Transaction a(p);
	ret = (*func)(&a, user_data);
	if (ret)
	    break;
    } // for

    return ret;
    }*/


