/***************************************************************************
  $RCSfile: dtausparser.cpp,v $
  -------------------
  cvs         : $Id: dtausparser.cpp,v 1.11 2004/07/19 21:50:42 rbla Exp $
  begin       : Wed Oct 22 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dtausparser.h"

#include <gwenhywfar/debug.h>
#include <openhbci2/openhbci2.h>
#include <openhbci2/value.h>
#include <openhbci2/datetime.h>
#include <openhbci2/hbcistring.h>
#include <cctype>

#include <gwenhywfar/text.h> // DEBUG

using namespace HBCI;


namespace HBCI {


DTAUSParser::DTAUSParser(){
}



DTAUSParser::~DTAUSParser(){
}


int DTAUSParser::_toDTA(int c) {
  if (isdigit(c))
    return c;
  if (c>='A' && c<='Z')
    return c;
  if (c>='a' && c<='z')
    return toupper(c);
  if (strchr(" .,&-+*%/$", c))
    return c;
  switch(c) {
  case '': return 0x5b;
  case '': return 0x5c;
  case '': return 0x5d;
  case '': return 0x7e;
  default:
    break;
  } /* switch */
  return 0;
}


int DTAUSParser::_fromDTA(int c) {
  if (isdigit(c))
    return c;
  if (c>='A' && c<='Z')
    return c;
  if (c>='a' && c<='z')
    return toupper(c);
  if (strchr(" .,&-+*%/$", c))
    return c;
  switch(c) {
  case 0x5b: return '';
  case 0x5c: return '';
  case 0x5d: return '';
  case 0x7e: return '?';
  default:
    break;
  } /* switch */
  return 0;
}


string DTAUSParser::_readWord(const string &s, unsigned int pos,
                              unsigned int size) {
  unsigned int i;
  string res;

  for (i=size; i>0; i--)
    if (!isspace(s[pos+i-1]))
      break;
  if (i) {
    size=i;
    for (i=0; i<size; i++) {
      char c;

      c=_fromDTA(s[pos+i]);
      if (c)
        res+=c;
      else
        res+=' ';
    } // for
    return res;
  }
  return "";
}


bool DTAUSParser::_parseExtSet(const string &d,
                               unsigned int pos,
                               GWEN_DB_NODE *xa){
  int typ;
  string tmp;

  if (pos+29>d.length()) {
    DBG_ERROR(0, "String too short");
    return false;
  }

  // read type
  if (1!=sscanf(d.substr(pos, 2).c_str(), "%d", &typ)) {
    DBG_ERROR(0, "not an integer");
    return false;
  }

  // read argument
  tmp=_readWord(d, pos+2, 27);
  if (tmp.empty()) {
    DBG_INFO(0, "Empty argument");
    return true;
  }
  switch(typ) {
  case 1: // Kundenname
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT, "othername", tmp.c_str());
    break;
  case 2: // Verwendungszweck
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT, "purpose", tmp.c_str());
    break;
  case 3: // Name des Auftraggebers
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT, "name", tmp.c_str());
    break;
  default: // unbekannt
    break;
  } // switch

  return true;
}



bool DTAUSParser::_parseSetA(const string &d,
                             unsigned int &pos,
                             GWEN_DB_NODE *xa){
  string tmp;

  if (pos+128>d.length()) {
    DBG_ERROR(0, "String too short");
    return false;
  }
  // transaction type
  tmp=_readWord(d, pos+5, 2);
  if (tmp=="GK")
    GWEN_DB_GroupRename(xa, "transaction");
  else if (tmp=="LK")
    GWEN_DB_GroupRename(xa, "debitnote");
  else {
    DBG_ERROR(0, "Transaction type \"%s\" not supported", tmp.c_str());
    return false;
  }
  // Bank code
  tmp=_readWord(d, pos+7, 8);
  GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT,
                       "ourAccount/bankCode",
                       tmp.c_str());
  // account id
  tmp=_readWord(d, pos+60, 10);
  GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT,
                       "ourAccount/accountId",
                       tmp.c_str());
  // customer reference
  tmp=_readWord(d, pos+70, 10);
  if (!tmp.empty())
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT,
                         "custref",
                         tmp.c_str());
  // execution date
  tmp=_readWord(d, pos+95, 8);
  if (!tmp.empty()) {
    if (tmp.length()!=8) {
      DBG_ERROR(0, "Invalid date");
      return false;
    }
    tmp=tmp.substr(4,4)+tmp.substr(2,2)+tmp.substr(0,2);
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT,
                         "date",
                         tmp.c_str());
  }
  if (tmp[pos+127]=='1')
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT,
                         "value/currency",
                         "EUR");
  else
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT,
                         "value/currency",
                         "DEM");
  pos+=128;
  return true;
}



bool DTAUSParser::_parseSetC(const string &d,
                             unsigned int &pos,
                             GWEN_DB_NODE *xa){
  string tmp;
  Value val;
  unsigned int extSets;
  unsigned int i;
  unsigned int lpos;
  double dd;

  if (pos+256>d.length()) {
    DBG_ERROR(0, "String too short");
    return false;
  }

  _cSets++;

  // Bank code
  tmp=_readWord(d, pos+13, 8);
  GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "otherAccount/bankCode",
                       tmp.c_str());
  dd=String::string2double(tmp.c_str());
  _sumBankCodes+=dd;

  // account id
  tmp=_readWord(d, pos+21, 10);
  GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "otherAccount/accountId",
                       tmp.c_str());
  dd=String::string2double(tmp.c_str());
  _sumAccountIds+=dd;

  // text key
  tmp=_readWord(d, pos+44, 2);
  GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT |
                       GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "textkey",
                       tmp.c_str());

  // Bank code
  tmp=_readWord(d, pos+61, 8);
  if (!tmp.empty())
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "ourAccount/bankCode",
                         tmp.c_str());
  // account id
  tmp=_readWord(d, pos+69, 10);
  if (!tmp.empty())
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "ourAccount/accountId",
                         tmp.c_str());

  // value
  if (d[pos+182]=='1') {
    double dd;

    // Euro
    tmp=_readWord(d, pos+79, 11);
    dd=String::string2double(tmp.c_str());
    dd=dd;
    try {
      val=Value(dd/100, "EUR");
    }
    catch (Error xerr) {
      DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
      return false;
    }
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "value/value",
                         val.toString(true).c_str());
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "value/currency", "EUR");
    _sumEUR+=dd;
  }
  else {
    double dd;

    // DM
    tmp=_readWord(d, pos+50, 11);
    dd=String::string2double(tmp.c_str());
    dd=dd;
    try {
      val=Value(dd/100, "DEM");
    }
    catch (Error xerr) {
      DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
      return false;
    }
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "value/value",
                         val.toString(true).c_str());
    GWEN_DB_SetCharValue(xa, GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "value/currency", "DEM");
    _sumDEM+=dd;
  }


  // othername
  tmp=_readWord(d, pos+93, 27);
  if (!tmp.empty())
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "othername",
                         tmp.c_str());

  // name
  tmp=_readWord(d, pos+128, 27);
  if (!tmp.empty())
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "name",
                         tmp.c_str());

  // purpose
  tmp=_readWord(d, pos+155, 27);
  if (!tmp.empty())
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "purpose",
                         tmp.c_str());

  // number of extension sets
  tmp=_readWord(d, pos+185, 2);
  if (1!=sscanf(tmp.c_str(), "%d", &extSets)) {
    DBG_ERROR(0, "not an integer");
    return false;
  }

  // read extension sets
  if (extSets>0) {
    if (!_parseExtSet(d, pos+187, xa)) {
      DBG_ERROR(0, "Error in first extension set");
      return false;
    }
  }

  if (extSets>1) {
    if (!_parseExtSet(d, pos+216, xa)) {
      DBG_ERROR(0, "Error in second extension set");
      return false;
    }
  }

  // read next extension sets
  i=2;
  lpos=256;
  while(i<extSets) {
    int j;

    for (j=0; j<4; j++) {
      if (i+j>=extSets)
        break;
      if (!_parseExtSet(d, pos+lpos, xa)) {
        DBG_ERROR(0, "Error in extension set %d", i+j);
        return false;
      }
      lpos+=29;
    } // for j
    i+=j;
    lpos+=12;
  } // while

  lpos=(lpos+127) & ~127;
  pos+=lpos;
  return true;
}



bool DTAUSParser::_parseSetE(const string &d,
                             unsigned int &pos){
  string tmp;
  unsigned int i;
  bool allOk;
  double dd;

  allOk=true;

  // number of C sets
  tmp=_readWord(d, pos+10, 7);
  if (1!=sscanf(tmp.c_str(), "%d", &i)) {
    DBG_ERROR(0, "not an integer");
    return false;
  }
  if (i!=_cSets) {
    DBG_ERROR(0, "Bad number of C sets (%d!=%d)", i, _cSets);
    allOk=false;
  }

  // checksum of DEM values
  tmp=_readWord(d, pos+17, 13);
  dd=String::string2double(tmp.c_str());
  if (dd!=_sumDEM) {
    DBG_ERROR(0, "Bad checksum for DEM values (%.2f!=%.2f)",
              (float)dd/100.0,
              (float)_sumDEM/100.0);
    allOk=false;
  }

  // checksum of account ids
  tmp=_readWord(d, pos+30, 17);
  dd=String::string2double(tmp.c_str());
  if (dd!=_sumAccountIds) {
    DBG_ERROR(0, "Bad checksum for account ids (%f!=%f)", dd, _sumAccountIds);
    allOk=false;
  }

  // checksum of bank codes
  tmp=_readWord(d, pos+47, 17);
  dd=String::string2double(tmp.c_str());
  if (dd!=_sumBankCodes) {
    DBG_ERROR(0, "Bad checksum for bank codes (%f!=%f)", dd, _sumBankCodes);
    allOk=false;
  }

  // checksum of EUR values
  tmp=_readWord(d, pos+64, 13);
  dd=String::string2double(tmp.c_str());
  if (dd!=_sumEUR) {
    DBG_ERROR(0, "Bad checksum for EUR values (%.2f!=%.2f)",
              (float)dd/100.0,
              (float)_sumEUR/100.0);
    return false;
  }
  pos+=128;
  if (allOk) {
    DBG_INFO(0, "DTAUS file ok");
  }
  return allOk;
}



bool DTAUSParser::parse(const string &d,
                        unsigned int &pos,
                        GWEN_DB_NODE *cfg){
  GWEN_DB_NODE *tmpl;
  GWEN_DB_NODE *xa;
  string tmp;

  // preset
  tmpl=0;
  _cSets=0;
  _sumEUR=0;
  _sumDEM=0;
  _sumBankCodes=0;
  _sumAccountIds=0;

  // read A set
  DBG_INFO(0, "Reading A set (pos=%d)", pos);
  if (d[pos+4]=='A') {
    // create template
    tmpl=GWEN_DB_Group_new("template");
    if (!_parseSetA(d, pos, tmpl)) {
      DBG_ERROR(0, "Error in A set");
      GWEN_DB_Group_free(tmpl);
      return false;
    }
  }

  // now read C sets
  while(pos<d.length()) {
    if (d[pos+4]=='C') {
      DBG_INFO(0, "Reading C set (pos=%d)", pos);
      if (tmpl)
        xa=GWEN_DB_Group_dup(tmpl);
      else
        xa=GWEN_DB_Group_new("transaction");
      if (!_parseSetC(d, pos, xa)) {
        DBG_ERROR(0, "Error in C set");
        GWEN_DB_Group_free(tmpl);
        return false;
      }
      GWEN_DB_AddGroup(cfg, xa);
    } // if C set
    else if (d[pos+4]=='E') {
      DBG_INFO(0, "Reading E set (pos=%d)", pos);
      // E set, check
      if (!_parseSetE(d, pos)) {
        DBG_ERROR(0, "Error in E set");
        GWEN_DB_Group_free(tmpl);
        return false;
      }
      break;
    } // if E set
    else {
      DBG_ERROR(0, "Unknown set \"%c\" at %d", d[pos+4], pos+4);
      GWEN_DB_Group_free(tmpl);
      return false;
    }
  } // while
  GWEN_DB_Group_free(tmpl);
  return true;
}



bool DTAUSParser::_writeWord(char *buffer,
                             unsigned int pos,
                             unsigned int size,
                             const string &p) {
  unsigned int i;
  unsigned int j;

  j=p.length();
  if (j>size) {
    DBG_WARN(0, "Word too long, chopping it \"%s\" (%d>%d)",
             p.c_str(), j, size);
  }

  for (i=0; i<size; i++) {
    char c;

    if (i>=j)
      c=0;
    else
      c=_toDTA(p[i]);
    if (c)
      buffer[pos+i]=c;
    else
      buffer[pos+i]=' ';
  } // for
  return true;
}



bool DTAUSParser::_writeNum(char *buffer,
                            unsigned int pos,
                            unsigned int size,
                            const string &p) {
  unsigned int i, j;

  i=p.length();
  if (i>size) {
    DBG_WARN(0, "Word too long, chopping it");
    i=size;
  }
  j=size-i;
  if (j) {
    unsigned int k;

    for (k=0; k<j; k++)
      buffer[pos+k]='0';
  }
  memmove(buffer+pos+j, p.c_str(), i);
  return true;
}



string DTAUSParser::_createSetA(GWEN_DB_NODE *cfg){
  char buffer[128];
  const char *p;
  DateTime dt;

  memset(buffer, 32, sizeof(buffer));
  buffer[0]='0';
  buffer[1]='1';
  buffer[2]='2';
  buffer[3]='8';
  buffer[4]='A';
  if (strcasecmp(GWEN_DB_GroupName(cfg), "transfer")==0 ||
      strcasecmp(GWEN_DB_GroupName(cfg), "transaction")==0)
    buffer[5]='G';
  else if (strcasecmp(GWEN_DB_GroupName(cfg), "debitnote")==0)
    buffer[5]='L';
  else {
    DBG_ERROR(0, "Unknown group \"%s\"", GWEN_DB_GroupName(cfg));
    return "";
  }
  buffer[6]='K';
  // our bank code
  if (!_writeNum(buffer, 7, 8,
		 GWEN_DB_GetCharValue(cfg, "ourAccount/bankCode", 0, "")))
    return "";
  // 000
  if (!_writeNum(buffer, 15, 8, "0"))
    return "";
  // our name
  p=GWEN_DB_GetCharValue(cfg, "name", 0, "");
  if (!*p) {
    DBG_WARN(0,
	     "Name of customer not given.\n"
	     "This may lead to extra charges (depending on your bank)");
  }
  if (!_writeWord(buffer, 23, 27, p))
    return "";
  // current date
  dt=DateTime::currentTime();
  if (!_writeWord(buffer, 50, 6, dt.timeString("%D%N%y")))
    return "";
  // our account id
  if (!_writeNum(buffer, 60, 10,
		 GWEN_DB_GetCharValue(cfg, "ourAccount/accountId", 0, "")))
    return "";
  // customer reference
  if (!_writeWord(buffer, 70, 10,
                  GWEN_DB_GetCharValue(cfg, "custref", 0, "")))
    return "";
  // date of execution
  p=GWEN_DB_GetCharValue(cfg, "date", 0, 0);
  if (p) {
    string ds;

    ds=p;
    if (!_writeWord(buffer, 95, 8,
		    ds.substr(6, 2)+ds.substr(4,2)+ds.substr(0, 4)))
      return "";
  }
  // currency mark
  p=GWEN_DB_GetCharValue(cfg, "value/currency", 0, 0);
  if (!p) {
    DBG_ERROR(0, "no currency");
    return "";
  }
  if (strcasecmp(p, "EUR")==0)
    buffer[127]='1';
  else if (strcasecmp(p, "DEM")!=0) {
    DBG_ERROR(0, "Bad currency \"%s\"", p);
    return "";
  }

  return string(buffer, 128);
}




string DTAUSParser::_createSetE(){
  char buffer[128];
  char numbuffer[32];

  memset(buffer, 32, sizeof(buffer));
  buffer[0]='0';
  buffer[1]='1';
  buffer[2]='2';
  buffer[3]='8';
  buffer[4]='E';

  sprintf(numbuffer, "%07d", _cSets);
  if (!_writeNum(buffer, 10, 7, numbuffer))
    return "";

  sprintf(numbuffer, "%013.0f", _sumDEM);
  if (!_writeWord(buffer, 17, 13, numbuffer))
    return "";

  sprintf(numbuffer, "%017.0f", _sumAccountIds);
  if (!_writeWord(buffer, 30, 17, numbuffer))
    return "";

  sprintf(numbuffer, "%017.0f", _sumBankCodes);
  if (!_writeWord(buffer, 47, 17, numbuffer))
    return "";

  sprintf(numbuffer, "%013.0f", _sumEUR);
  if (!_writeWord(buffer, 64, 13, numbuffer))
    return "";

  return string(buffer, 128);
}



string DTAUSParser::_createSetC(GWEN_DB_NODE *cfg){
  char buffer[622];
  char numbuffer[32];
  int extSets;
  double dd;
  string tmp;
  unsigned int i;
  unsigned int size;
  unsigned int realSize;

  extSets=0;

  // preset
  memset(buffer, 32, sizeof(buffer));

  // our bank code
  tmp=GWEN_DB_GetCharValue(cfg, "ourAccount/bankCode", 0, "");
  if (!_writeNum(buffer, 5, 8, tmp))
    return "";

  // other bank code
  tmp=GWEN_DB_GetCharValue(cfg, "otherAccount/bankCode", 0, "");
  dd=String::string2double(tmp.c_str());
  _sumBankCodes+=dd;
  if (!_writeNum(buffer, 13, 8, tmp))
    return "";

  // other account id
  tmp=GWEN_DB_GetCharValue(cfg, "otherAccount/accountid", 0, "");
  dd=String::string2double(tmp.c_str());
  _sumAccountIds+=dd;
  if (!_writeNum(buffer, 21, 10, tmp))
    return "";

  // "interne Kundennummer"
  if (!_writeNum(buffer, 31, 13, "0"))
    return "";

  // text key
  tmp=String::num2string(GWEN_DB_GetIntValue(cfg, "textkey", 0, 51),
                         true, 2);
  if (!_writeNum(buffer, 44, 2, tmp))
    return "";

  // 7b
  if (!_writeNum(buffer, 46, 3, "0"))
    return "";

  // value
  dd=String::string2double(GWEN_DB_GetCharValue(cfg,
                                                "value/value", 0,
                                                "0"))*100;
  sprintf(numbuffer, "%011.0f", dd);
  if (strcasecmp(GWEN_DB_GetCharValue(cfg, "value/currency", 0, "EUR"),
		 "EUR")==0) {
    // EUR
    buffer[182]='1';
    _sumEUR+=dd;
    if (!_writeNum(buffer, 79, 11, numbuffer))
      return "";
    if (!_writeNum(buffer, 50, 11, "0"))
      return "";
  }
  else {
    // DEM
    buffer[182]=' ';
    _sumDEM+=dd;
    if (!_writeNum(buffer, 79, 11, "0"))
      return "";
    if (!_writeNum(buffer, 50, 11, numbuffer))
      return "";
  }

  // our bank code
  if (!_writeNum(buffer, 61, 8,
		 GWEN_DB_GetCharValue(cfg, "ourAccount/bankCode", 0, "")))
    return "";

  // our account id
  if (!_writeNum(buffer, 69, 10,
		 GWEN_DB_GetCharValue(cfg, "ourAccount/accountid", 0, "")))
    return "";

  // name
  if (!_writeWord(buffer, 128, 27,
                  GWEN_DB_GetCharValue(cfg, "name", 0, "")))
    return "";

  // othername
  for (i=0; ; i++) {
    const char *p;

    p=GWEN_DB_GetCharValue(cfg, "othername", i, 0);
    if (!p) {
      if (!i) {
        DBG_WARN(0, "No othername given");
      }
      break;
    }
    if (i==0) {
      // store name in first possible field
      if (!_writeWord(buffer, 93, 27, p))
        return "";
    }
    else {
      if (extSets==0) {
        // store name in first extension
        buffer[187]='0';
        buffer[188]='1';
        if (!_writeWord(buffer, 189, 27, p))
          return "";
        extSets++;
      }
      else if (extSets==1) {
        // store name in second extension
        buffer[216]='0';
        buffer[217]='1';
        if (!_writeWord(buffer, 218, 27, p))
          return "";
        extSets++;
      }
      else {
        // we need to actually create an extensions
        unsigned int lpos;

        lpos=(((extSets-2)/4)*128)+((extSets-2)%4)*29;
        buffer[256+lpos]='0';
        buffer[256+lpos+1]='1';
        if (!_writeWord(buffer, 256+lpos+2, 27, p))
          return "";
        extSets++;
      }
    } // if extension needed
  } // for

  // description
  for (i=0; ; i++) {
    const char *p;

    p=GWEN_DB_GetCharValue(cfg, "purpose", i, 0);
    if (!p) {
      if (!i) {
        DBG_ERROR(0, "No purpose given");
        return "";
      }
      break;
    }
    if (i==0) {
      // store purpose in first possible field
      if (!_writeWord(buffer, 155, 27, p))
        return "";
    }
    else {
      if (extSets==0) {
        // store name in first extension
        buffer[187]='0';
        buffer[188]='2';
        if (!_writeWord(buffer, 189, 27, p))
          return "";
        extSets++;
      }
      else if (extSets==1) {
        // store name in second extension
        buffer[216]='0';
        buffer[217]='2';
        if (!_writeWord(buffer, 218, 27, p))
          return "";
        extSets++;
      }
      else {
        // we need to actually create an extensions
        unsigned int lpos;

        lpos=(((extSets-2)/4)*128)+((extSets-2)%4)*29;
        buffer[256+lpos]='0';
        buffer[256+lpos+1]='2';
        if (!_writeWord(buffer, 256+lpos+2, 27, p))
          return "";
        extSets++;
      }
    } // if extension needed
  } // for

  // calculate size
  size=187+(extSets*29);
  sprintf(numbuffer, "%04d", size);
  if (!_writeNum(buffer, 0, 4, numbuffer))
    return "";
  buffer[4]='C';
  // store number of ext sets
  sprintf(numbuffer, "%02d", extSets);
  if (!_writeNum(buffer, 185,  2, numbuffer))
    return "";

  if (extSets<=2)
    realSize=256;
  else {
    realSize=256+
      ((((((extSets-2)/4)*128)+((extSets-2)%4)*29)+127)&~127);
  }

  _cSets++;
  return string(buffer, realSize);
}



string DTAUSParser::create(GWEN_DB_NODE *cfg){
  string result;
  GWEN_DB_NODE *n;
  unsigned int i;
  const char *p;
  string part;

  _cSets=0;
  _sumEUR=0;
  _sumDEM=0;
  _sumBankCodes=0;
  _sumAccountIds=0;

  i=0;
  n=GWEN_DB_GetFirstGroup(cfg);
  if (n) {
    p=GWEN_DB_GroupName(n);
    while (n) {
      part.erase();
      if (strcasecmp(p, GWEN_DB_GroupName(n))==0) {
        if (i==0) {
          DBG_INFO(0, "Creating A set");
          // create A set (if i==0)
          part=_createSetA(n);
          if (part.empty())
            return "";
          result+=part;
          part.erase();
        }
        DBG_INFO(0, "Creating C set");
        part=_createSetC(n);
        if (part.empty())
          return "";
        result+=part;
      } // if matching groupname
      else {
        DBG_ERROR(0, "Mixing is not allowed (\"%s\")", p);
      }
      n=GWEN_DB_GetNextGroup(n);
      i++;
    } // while

    // create E set
    DBG_INFO(0, "Creating E set");
    part=_createSetE();
    if (part.empty())
      return "";
    result+=part;
  } // if transactions

  return result;
}









}; // namespace HBCI



