/***************************************************************************
  $RCSfile: dtausparser.h,v $
  -------------------
  cvs         : $Id: dtausparser.h,v 1.3 2004/01/19 20:20:35 aquamaniac Exp $
  begin       : Wed Oct 22 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DTAUSPARSER_H
#define DTAUSPARSER_H

#include <gwenhywfar/db.h>
#include <openhbci2/openhbci2.h>

#include <string>

using namespace std;


namespace HBCI {

/**
 * This class implements a parser for DTAUS files.
 * It allows parsing and creating of such data.
 * @author Martin Preuss<martin@libchipcard.de>
 */
class DTAUSParser {
private:

  unsigned int _cSets;
  double _sumEUR;
  double _sumDEM;
  double _sumBankCodes;
  double _sumAccountIds;

  string _readWord(const string &s, unsigned int pos,
                   unsigned int size);
  bool _parseExtSet(const string &d, unsigned int pos, GWEN_DB_NODE *xa);
  bool _parseSetA(const string &d, unsigned int &pos, GWEN_DB_NODE *xa);
  bool _parseSetC(const string &d, unsigned int &pos, GWEN_DB_NODE *xa);
  bool _parseSetE(const string &d, unsigned int &pos);

  bool _writeWord(char *buffer,
                  unsigned int pos,
                  unsigned int size,
                  const string &p);
  bool _writeNum(char *buffer,
                 unsigned int pos,
                 unsigned int size,
                 const string &p);
  int _toDTA(int c);
  int _fromDTA(int c);

  string _createSetA(GWEN_DB_NODE *cfg);
  string _createSetC(GWEN_DB_NODE *cfg);
  string _createSetE();

public:
  DTAUSParser();
  ~DTAUSParser();
  bool parse(const string &d, unsigned int &pos, GWEN_DB_NODE *cfg);
  string create(GWEN_DB_NODE *cfg);

};

}; // namespace HBCI



#endif


