/***************************************************************************
  $RCSfile: dtausplugin.cpp,v $
  -------------------
  cvs         : $Id: dtausplugin.cpp,v 1.4 2003/11/22 20:09:00 aquamaniac Exp $
  begin       : Wed Oct 22 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "dtausplugin.h"
#include "dtausparser.h"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>


namespace HBCI {

  DtausParserPlugin::DtausParserPlugin(Hbci *hbci)
  :ParserPlugin(hbci){
  }



  DtausParserPlugin::~DtausParserPlugin(){
  }



  int DtausParserPlugin::readBinType(GWEN_XMLNODE *node,
                                     GWEN_DB_NODE *gr,
                                     GWEN_BUFFER *gbuf) {
    const char *type;
    const char *subtype;
    string datastr;
    unsigned int pos;
    GWEN_DB_NODE *mtgrp;
    const char *name;

    name=GWEN_XMLNode_GetProperty(node, "name","");
    type=GWEN_XMLNode_GetProperty(node, "bintype","");
    subtype=GWEN_XMLNode_GetProperty(node, "binsubtype","");
    if (strcasecmp(type, "dtaus")) {
      DBG_ERROR(0,"Unsupported type (\"%s\" is not DTAUS)", type);
      return 1;
    }

    datastr=string(GWEN_Buffer_GetStart(gbuf),
                   GWEN_Buffer_GetUsedBytes(gbuf));

    pos=0;
    mtgrp=GWEN_DB_GetGroup(gr, GWEN_DB_FLAGS_DEFAULT, name);
    assert(mtgrp);
    while (pos<datastr.length()) {
      if (!_parser.parse(datastr, pos, mtgrp)) {
        return 1;
      }
    } // while

    return 0;
  }



  int DtausParserPlugin::writeBinType(GWEN_XMLNODE *node,
                                      GWEN_DB_NODE *gr,
                                      GWEN_BUFFER *gbuf) {
    string result;
    const char *type;
    const char *subtype;
    string datastr;
    GWEN_DB_NODE *mtgrp;
    const char *name;

    name=GWEN_XMLNode_GetProperty(node, "name","");
    type=GWEN_XMLNode_GetProperty(node, "bintype","");
    subtype=GWEN_XMLNode_GetProperty(node, "binsubtype","");
    if (strcasecmp(type, "dtaus")) {
      DBG_ERROR(0,"Unsupported type (\"%s\" is not DTAUS)", type);
      return 1;
    }

    mtgrp=GWEN_DB_GetGroup(gr, GWEN_DB_FLAGS_DEFAULT, name);
    assert(mtgrp);
    result=_parser.create(mtgrp);
    if (result.empty())
      return 1;
    if (GWEN_Buffer_AppendBytes(gbuf, result.data(), result.length())) {
      DBG_INFO(0, "called from here");
      return 1;
    }
    return 0;
  }


}; // namespace




/* Plugin handling */

HBCI::Error dtaus_checkVersion() {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  HBCI::Hbci::libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile DTAUS plugin.\n");
    return HBCI::Error("DTAUS Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v%d.%d.%d.%d or newer.\n",
	    OPENHBCI_VERSION_MAJOR,
	    OPENHBCI_VERSION_MINOR,
	    OPENHBCI_VERSION_PATCHLEVEL,
	    OPENHBCI_VERSION_BUILD);
    return HBCI::Error("DTAUS Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }
  return HBCI::Error();
}

extern "C" {
  // The version of openhbci's plugin interface system (i.e. the
  // convention that this plugin has to define a registerYourSelf()
  // function.)
  int dtaus_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  // Version numbers of this plugin
  int dtaus_openhbci_module_current=DTAUS_MODULE_CURRENT;
  int dtaus_openhbci_module_revision=DTAUS_MODULE_REVISION;
  int dtaus_openhbci_module_age=DTAUS_MODULE_AGE;


  // The function that registers this plugin at the HBCI::API. This
  // function will be looked up by dlsym() and called from the plugin
  // interface system.
  HBCI::Error dtaus_registerPlugin(HBCI::Hbci *hbci,
				   HBCI::PluginManager *mgr,
				   PluginType pt) {
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    if (pt!=PluginTypeParser) {
      return HBCI::Error("DTAUS Plugin",
			 ERROR_LEVEL_NORMAL,
			 HBCI_ERROR_CODE_MEDIUM,
			 ERROR_ADVISE_DONTKNOW,
			 "bad plugin type");
    }
    err=dtaus_checkVersion();
    if (!err.isOk())
      return HBCI::Error("DTAUS Plugin",err);

    mp=new HBCI::DtausParserPlugin(hbci);
    return mgr->registerPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::Plugin> dtaus_createPlugin(HBCI::Hbci *hbci,
						 HBCI::PluginManager *mgr){
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=dtaus_checkVersion();
    if (!err.isOk())
      throw HBCI::Error("DTAUS Plugin",err);

    mp=new HBCI::DtausParserPlugin(hbci);
    mp.setDescription("DTAUSPlugin");
    return mp;
  }
} // extern "C"








