/***************************************************************************
  $RCSfile: msgenginehbci.c,v $
  -------------------
  cvs         : $Id: msgenginehbci.c,v 1.7 2004/05/27 21:45:36 aquamaniac Exp $
  begin       : Mon Jul 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "msgenginehbci_p.h"
#include <gwenhywfar/debug.h>
#include <gwenhywfar/inherit.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>


GWEN_INHERIT(GWEN_MSGENGINE, HBCI_MessageEngineXML)


/* --------------------------------------------------------------- FUNCTION */
int MsgEngineHBCI_ReadBinType(GWEN_MSGENGINE *e,
                              GWEN_XMLNODE *node,
                              GWEN_DB_NODE *gr,
                              GWEN_BUFFER *dbuf){
  HBCI_MessageEngineXML *x;

  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return 1;
  return HBCI_MsgEngineXML_ReadBinType(x, node, gr, dbuf);
}



int MsgEngineHBCI_WriteBinType(GWEN_MSGENGINE *e,
                               GWEN_XMLNODE *node,
                               GWEN_DB_NODE *gr,
                               GWEN_BUFFER *dbuf) {
  HBCI_MessageEngineXML *x;

  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return 1;
  return HBCI_MsgEngineXML_WriteBinType(x, node, gr, dbuf);
}






int MsgEngineHBCI_TypeRead(GWEN_MSGENGINE *e,
                           GWEN_BUFFER *msgbuf,
                           GWEN_XMLNODE *node,
                           GWEN_BUFFER *vbuf,
                           char escapeChar,
                           const char *delimiters){
  HBCI_MessageEngineXML *x;

  DBG_DEBUG(0, "MsgEngineHBCI_TypeRead");
  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return 1;

  return HBCI_MsgEngineXML_TypeRead(x, msgbuf, node, vbuf, escapeChar,
                                    delimiters);
}



int MsgEngineHBCI_TypeWrite(GWEN_MSGENGINE *e,
                            GWEN_BUFFER *gbuf,
                            GWEN_BUFFER *data,
                            GWEN_XMLNODE *node){
  HBCI_MessageEngineXML *x;

  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return 1;

  return HBCI_MsgEngineXML_TypeWrite(x, gbuf, data, node);
}



GWEN_DB_VALUETYPE MsgEngineHBCI_TypeCheck(GWEN_MSGENGINE *e,
					  const char *tname){
  HBCI_MessageEngineXML *x;

  DBG_DEBUG(0, "MsgEngineHBCI_TypeCheck");
  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return GWEN_DB_VALUETYPE_UNKNOWN;

  return HBCI_MsgEngineXML_TypeCheck(x, tname);
}



const char *MsgEngineHBCI_GetCharValue(GWEN_MSGENGINE *e,
				       const char *name,
				       const char *defValue){
  HBCI_MessageEngineXML *x;

  DBG_DEBUG(0, "MsgEngineHBCI_GetCharValue");
  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return defValue;

  return HBCI_MsgEngineXML_GetCharValue(x, name, defValue);
}



int MsgEngineHBCI_GetIntValue(GWEN_MSGENGINE *e,
                              const char *name,
                              int defValue){
  HBCI_MessageEngineXML *x;

  DBG_DEBUG(0, "MsgEngineHBCI_GetIntValue");
  x=GWEN_INHERIT_GETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML, e);
  if (!x)
    return defValue;

  return HBCI_MsgEngineXML_GetIntValue(x, name, defValue);
}



void MsgEngineHBCI_FreeData(void *bp, void *p) {

}



GWEN_MSGENGINE *MsgEngineHBCI_new(HBCI_MessageEngineXML *x){
  GWEN_MSGENGINE *e;

  e=GWEN_MsgEngine_new();
  assert(e);
  GWEN_INHERIT_SETDATA(GWEN_MSGENGINE, HBCI_MessageEngineXML,
                       e, x,
                       MsgEngineHBCI_FreeData);
  GWEN_MsgEngine_SetTypeReadFunction(e, MsgEngineHBCI_TypeRead);
  GWEN_MsgEngine_SetTypeWriteFunction(e, MsgEngineHBCI_TypeWrite);
  GWEN_MsgEngine_SetTypeCheckFunction(e, MsgEngineHBCI_TypeCheck);
  GWEN_MsgEngine_SetBinTypeReadFunction(e, MsgEngineHBCI_ReadBinType);
  GWEN_MsgEngine_SetBinTypeWriteFunction(e, MsgEngineHBCI_WriteBinType);
  GWEN_MsgEngine_SetGetCharValueFunction(e, MsgEngineHBCI_GetCharValue);
  GWEN_MsgEngine_SetGetIntValueFunction(e, MsgEngineHBCI_GetIntValue);
  GWEN_MsgEngine_SetEscapeChar(e, '?');
  return e;
}


void MsgEngineHBCI_free(GWEN_MSGENGINE *e){
  GWEN_MsgEngine_free(e);

}



