/***************************************************************************
  $RCSfile: msgxml.h,v $
  -------------------
  cvs         : $Id: msgxml.h,v 1.9 2004/01/19 20:20:35 aquamaniac Exp $
  begin       : Mon Jul 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef MSGXML_H
#define MSGXML_H

#define MSGXML_TYPENAME "msgxml"

#define MSGXML_MAXMSGSIZE 8192

#ifdef __cplusplus

namespace HBCI {
  class MessageEngineXML;
}
typedef struct HBCI::MessageEngineXML HBCI_MessageEngineXML;
#else
typedef struct HBCI_MessageEngineXML HBCI_MessageEngineXML;
#endif


#include <gwenhywfar/msgengine.h>


#ifdef __cplusplus

#include <openhbci2/pointer.h>
#include <openhbci2/msgengine.h>
#include <openhbci2/transporter.h>
#include "msgxmljob.h"
#include "msgxmlqueue.h"



namespace HBCI {

  class MessageEngineXML: public MessageEngine {
    friend class MsgXMLPlugin;
  private:

    class XMLFile {
    private:
      string _fpath;
      GWEN_XMLNODE *_node;
      int _counter;
    public:
      XMLFile(const string &path, GWEN_XMLNODE *n);
      ~XMLFile();
      const string &fileName() const { return _fpath;};
      GWEN_XMLNODE *node() { return _node;};
      void attach();
      void detach();
      unsigned int usedBy() const { return _counter;};
    };

    static list<Pointer<XMLFile> > _xmlFiles;
    GWEN_MSGENGINE *_engine;
    GWEN_DB_NODE *_config;
    int _msgTimeout;
    int _hversion;
    string _logDir;

    Pointer<Medium> _medium;
    bool _dialogOpen;
    string _logFile;
    Pointer<Customer> _sessionOwner;
    Pointer<Transporter> _sessionTransporter;

    GWEN_DB_NODE *_exchangeMessages(Pointer<Transporter> tx,
                                    Pointer<Customer> owner,
                                    list<Pointer<Customer> > signers,
                                    char *msg,
                                    unsigned int msgSize,
                                    bool sign,
                                    bool crypt);
    Error _handleQueue(Pointer<MsgQueueXML> q);
    Error _loadConfig(int country,
                      const string &bankCode,
                      int hversion,
                      const string &name);

    GWEN_DB_NODE *_handleJobs(Pointer<Transporter> tx,
			     Pointer<Customer> owner,
			     list<Pointer<Customer> > signers,
			     list<Pointer<MsgJobXML> > jobs);
    GWEN_DB_NODE *_handleDlgJob(Pointer<MsgJobXML> job,
                                GWEN_XMLNODE *node,
                                GWEN_DB_NODE *jobdata);
    bool _compareSignatures(const list<Pointer<Customer> > &s,
			    const list<Pointer<Customer> > &s2);

    Error _prepareSignatures(GWEN_DB_NODE *cfg);

    GWEN_DB_NODE *_openDialog();
    Error _closeDialog();
    Pointer<MsgJobXML> _jobFactory(const string &name,
				   Pointer<Customer> cu,
				   const string &account="");

    void _setBinProperty(GWEN_DB_NODE *cfg,
			 const string &name,
			 const string &value);
    string _getBinProperty(GWEN_DB_NODE *cfg,
			   const string &name,
			   const string &defValue);
    Error _mountCustomersMedium(Pointer<Customer> cust);
    string _encode(char *msg,
		   unsigned int msgSize,
		   Pointer<Customer> owner,
		   const list<Pointer<Customer> > &signers,
		   bool needSign,
		   bool needCrypt);
    GWEN_DB_NODE *_decode(Pointer<Customer> owner,
			 string data);

    string _readMessage(Pointer<Transporter> tx);

    Error _openConnection(Pointer<Transporter> tx);

    Error _processResponse(Pointer<MsgQueueXML> mq,
			   list<Pointer<MsgJobXML> > &jobs,
			   GWEN_DB_NODE *responseData);
    Error _openSendClose(Pointer<MsgQueueXML> q,
                         const list<Pointer<Customer> > &signers,
			 list<Pointer<MsgJobXML> > &jobs);

    void _cleanUpConfig(GWEN_DB_NODE *cfg, const char *name);

    string _hash(const string &s);
    int _toDTA(int c);
    int _fromDTA(int c);
    void _logMessage(const char *msg,
                     unsigned int msgSize,
                     bool crypt,
                     bool bank);


    // for MsgXMLPlugin
    GWEN_DB_NODE *config() { return _config;};
    MessageEngineXML(Hbci *hbci,
		     int country,
		     const string &bankCode,
                     int hversion,
		     GWEN_DB_NODE *cfg);

  public:
    MessageEngineXML(Hbci *hbci,
		     int country,
		     const string &bankCode,
		     int hversion);
    virtual ~MessageEngineXML();

    string engineType() const { return MSGXML_TYPENAME;};
    const string &logDir() const { return _logDir;};

    void setLogDir(const string &ld) { _logDir=ld;};

    /**
     * @internal
     * Call this before executing queues with this engine.
     * After executing the last queue you should call @ref endSession to
     * free ressources.
     */
    Error startSession(Pointer<Customer> owner,
                       Pointer<Transporter> tx);

    /**
     * @internal
     * Call this after executing queues with this engine.
     */
    Error endSession();
    Error executeQueue(Pointer<MsgQueue> mq);

    void setHbciVersion(int hversion);

    /**
     * Creates a MsgJob object. This way OpenHBCI is fully independant of
     * the actual implementation of the MsgJob.
     */
    virtual Pointer<MsgJob> jobFactory(const string &name,
				       Pointer<Customer> cu,
				       const string &account="");

    /**
     * Creates MsgQueue object. This way OpenHBCI is fully independant of
     * the actual implementation of the MsgQueue.
     */
    virtual Pointer<MsgQueue> queueFactory(Pointer<Customer> cu);


    virtual int readBinType(GWEN_XMLNODE *node,
                            GWEN_DB_NODE *gr,
                            GWEN_BUFFER *dbuf);

    virtual int writeBinType(GWEN_XMLNODE *node,
                             GWEN_DB_NODE *gr,
                             GWEN_BUFFER *dbuf);

    virtual int typeRead(GWEN_BUFFER *msgbuf,
                         GWEN_XMLNODE *node,
                         GWEN_BUFFER *vbuf,
                         char escapeChar,
                         const char *delimiters);
    virtual int typeWrite(GWEN_BUFFER *gbuf,
                          GWEN_BUFFER *data,
                          GWEN_XMLNODE *node);
    virtual GWEN_DB_VALUETYPE typeCheck(const char *name);
    virtual const char *getCharValue(const char *name, const char *defValue);
    virtual int getIntValue(const char *name, int defValue);


  }; /* class */
}; /* namespace */


#endif /* #ifdef __cplusplus */


#ifdef __cplusplus
extern "C" {
#endif

  extern int HBCI_MsgEngineXML_ReadBinType(HBCI_MessageEngineXML *e,
                                           GWEN_XMLNODE *node,
                                           GWEN_DB_NODE *gr,
                                           GWEN_BUFFER *dbuf);

  extern int HBCI_MsgEngineXML_WriteBinType(HBCI_MessageEngineXML *e,
					    GWEN_XMLNODE *node,
                                            GWEN_DB_NODE *gr,
                                            GWEN_BUFFER *dbuf);

  extern int HBCI_MsgEngineXML_TypeRead(HBCI_MessageEngineXML *e,
                                        GWEN_BUFFER *msgbuf,
                                        GWEN_XMLNODE *node,
                                        GWEN_BUFFER *vbuf,
                                        char escapeChar,
                                        const char *delimiters);
  extern int HBCI_MsgEngineXML_TypeWrite(HBCI_MessageEngineXML *e,
                                         GWEN_BUFFER *gbuf,
                                         GWEN_BUFFER *data,
                                         GWEN_XMLNODE *node);
  extern GWEN_DB_VALUETYPE
    HBCI_MsgEngineXML_TypeCheck(HBCI_MessageEngineXML *e,
                                const char *tname);
  extern const char *HBCI_MsgEngineXML_GetCharValue(HBCI_MessageEngineXML *e,
                                                    const char *name,
                                                    const char *defValue);
  extern int HBCI_MsgEngineXML_GetIntValue(HBCI_MessageEngineXML *e,
                                           const char *name,
                                           int defValue);


#ifdef __cplusplus
}
#endif




#endif



