/***************************************************************************
  $RCSfile: msgxmljob.h,v $
  -------------------
  cvs         : $Id: msgxmljob.h,v 1.7 2004/01/19 20:20:35 aquamaniac Exp $
  begin       : Mon Jul 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MSGXMLJOB_H
#define MSGXMLJOB_H

#ifdef __cplusplus
namespace HBCI{
  class MsgJobXML;
};

#endif

#include <openhbci2.h>
#include "msgxml.h"


namespace HBCI{

  /**
   * @author Martin Preuss<martin@libchipcard.de>
   */
  class MsgJobXML: public MsgJob {
  private:
    MessageEngineXML *_engine;
    GWEN_DB_NODE *_jobData;
    GWEN_XMLNODE *_node;
    string _secmode;
    list<Result> _results;
    MessageReference _msgRef;
    list<Pointer<Customer> > _signers;
    int _firstseg;
    int _lastseg;
    int _signsNeeded;
    int _jobsPerMessage;
    bool _attachable;
    bool _single;
    bool _dlg;
    bool _needresponse;
    string _description;

  public:
    MsgJobXML(MessageEngineXML *engine,
	      Pointer<Customer> cu,
	      const string &name, const string &accountCode,
	      const string &secmode,
	      GWEN_DB_NODE *cfg,
	      GWEN_XMLNODE *node);
    ~MsgJobXML();

    virtual bool single() { return _single;};
    virtual int jobsPerMessage() { return _jobsPerMessage;};
    /** returns true if the job may have  an attachment point */
    virtual bool attachable() { return _attachable;};
    virtual bool needResponse() { return _needresponse;};
    virtual int signatureMinimum() { return _signsNeeded;};
    virtual string description() const {return _description;};

    const char *getProperty(const string &name,
			    const string &defValue);
    int getIntProperty(const string &name,
		       int defValue);
    string getBinProperty(const string &name,
			  const string &defValue);
    void setProperty(const string &name, const string &value);
    void setIntProperty(const string &name, int value);
    void setBinProperty(const string &name, const string &value);
    void addRequestData(const string &path, GWEN_DB_NODE *gr);

    bool needsSignature() const;
    bool needsEncryption() const;
    const list<Result> &results() const;
    void removeResults(unsigned int code);
    void addSigner(Pointer<Customer> cu);
    const list<Pointer<Customer> > &signers() const;

    const MessageReference &messageReference() const { return _msgRef;};

    void setMessageReference(const MessageReference &r) { _msgRef=r;};

    GWEN_DB_NODE *jobData() { return _jobData;};
    GWEN_XMLNODE *node() { return _node;};
    const string &securityMode() const { return _secmode;};
    void setFirstSegment(int i) { _firstseg=i;};
    void setLastSegment(int i) { _lastseg=i;};
    bool hasSegment(int i);
    void addResult(const Result &r);
  };


};




#endif


