/***************************************************************************
  $RCSfile: msgxmlplugin.cpp,v $
  -------------------
  cvs         : $Id: msgxmlplugin.cpp,v 1.5 2004/01/19 20:20:35 aquamaniac Exp $
  begin       : Mon Jul 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/



#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "openhbci2.h"
#include "msgxmlplugin.h"
#include "msgxml.h"
#include "hbcistring.h"
#include <string.h>
#include <openhbci2/pluginmanager.h>
#include <gwenhywfar/directory.h>

namespace HBCI {



MsgXMLPlugin::MsgXMLPlugin(Hbci *hbci)
    :MessageEnginePlugin(hbci)
{

}


MsgXMLPlugin::~MsgXMLPlugin(){
}


Pointer<MessageEngine>
MsgXMLPlugin::engineFromConfig(int country,
			       const string &bankCode,
			       int hversion,
			       GWEN_DB_NODE *cfg) {
  string typ;
  Pointer<MessageEngineXML> mkf;

  if (!cfg) {
    throw Error("MsgXMLPlugin::engineFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"no configuration given");
  }
  typ=GWEN_DB_GetCharValue(cfg, "enginetype",0, "");
  if (strcasecmp(typ.c_str(), pluginName().c_str())!=0) {
    throw Error("MsgXMLPlugin::engineFromConfig",
		ERROR_LEVEL_NORMAL,
		HBCI_ERROR_CODE_INVALID,
		ERROR_ADVISE_DONTKNOW,
		"bad engine type");
  }

  mkf=new MessageEngineXML(hbci(), country, bankCode, hversion, cfg);
  return mkf.cast<MessageEngine>();
}


Error MsgXMLPlugin::engineToConfig(Pointer<MessageEngine> m,
				   GWEN_DB_NODE *cfg) {
  Pointer<MessageEngineXML> mkf;

  try {
    mkf=m.cast<MessageEngineXML>();
  }
  catch (Error xerr) {
    return Error("MsgXMLPlugin::engineToConfig",xerr);
  }
  GWEN_DB_AddGroupChildren(cfg, mkf.ref().config());
  GWEN_DB_SetCharValue(cfg,
                       GWEN_DB_FLAGS_DEFAULT | GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "enginetype",
                       pluginName().c_str());
  GWEN_DB_SetCharValue(cfg,
                       GWEN_DB_FLAGS_DEFAULT | GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "logdir",
                       mkf.ref().logDir().c_str());

  return Error();
}


Pointer<MessageEngine> MsgXMLPlugin::engineFactory(int country,
						   const string &bankCode,
						   int hversion){
  Pointer<MessageEngine> p;

  p=new MessageEngineXML(hbci(), country, bankCode, hversion);
  p.setObjectDescription("MsgXMLPlugin::engineFactory: MessageEngine");
  p.setDescription("MsgXMLPlugin::engineFactory: MessageEngine");
  return p;
}




}; // namespace


HBCI::Error msgxml_checkVersion() {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  HBCI::API::libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile RDHFile plugin.\n");
    return HBCI::Error("MsgXML Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v%d.%d.%d.%d or newer.\n",
	    OPENHBCI_VERSION_MAJOR,
	    OPENHBCI_VERSION_MINOR,
	    OPENHBCI_VERSION_PATCHLEVEL,
	    OPENHBCI_VERSION_BUILD);
    return HBCI::Error("RDHFile Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }
  return HBCI::Error();
}


extern "C" {
  // The version of openhbci's plugin interface system (i.e. the
  // convention that this plugin has to define a registerYourSelf()
  // function.)
  int msgxml_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  // Version numbers of this plugin
  int msgxml_openhbci_module_current=MSGXML_MODULE_CURRENT;
  int msgxml_openhbci_module_revision=MSGXML_MODULE_REVISION;
  int msgxml_openhbci_module_age=MSGXML_MODULE_AGE;


  // The function that registers this plugin at the HBCI::API. This
  // function will be looked up by dlsym() and called from the plugin
  // interface system.
  HBCI::Error msgxml_registerPlugin(HBCI::Hbci *hbci,
				     HBCI::PluginManager *mgr,
				     PluginType pt) {
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    if (pt!=PluginTypeMessage) {
      return HBCI::Error("RDHFile Plugin",
			 ERROR_LEVEL_NORMAL,
			 HBCI_ERROR_CODE_MEDIUM,
			 ERROR_ADVISE_DONTKNOW,
			 "bad plugin type");
    }
    err=msgxml_checkVersion();
    if (!err.isOk())
      return HBCI::Error("MsgXML Plugin",err);

    mp=new HBCI::MsgXMLPlugin(hbci);
    return mgr->registerPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::Plugin> msgxml_createPlugin(HBCI::Hbci *hbci,
						   HBCI::PluginManager *mgr){
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=msgxml_checkVersion();
    if (!err.isOk())
      throw HBCI::Error("MsgXML Plugin",err);

    mp=new HBCI::MsgXMLPlugin(hbci);
    mp.setDescription("MsgXMLPlugin");
    return mp;
  }


}; // extern "C"


