/***************************************************************************
  $RCSfile: msgxmlplugin.h,v $
  -------------------
  cvs         : $Id: msgxmlplugin.h,v 1.2 2004/01/19 20:20:35 aquamaniac Exp $
  begin       : Mon Jul 07 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef MSGXMLPLUGIN_H
#define MSGXMLPLUGIN_H



#include <openhbci2/msgengineplugin.h>

#include "msgxml.h"

#define MSGXML_PLUGIN_VMAJOR 1
#define MSGXML_PLUGIN_VMINOR 0

#define MSGXML_MODULE_CURRENT  0
#define MSGXML_MODULE_REVISION 0
#define MSGXML_MODULE_AGE      0

#ifdef __cplusplus
namespace HBCI {



/**
 * This is the plugin for RDH keyfiles in OpenHBCI's own format.
 * @author Martin Preuss<martin@libchipcard.de>
 */
class MsgXMLPlugin: public MessageEnginePlugin {
public:
  MsgXMLPlugin(Hbci *hbci);
  virtual ~MsgXMLPlugin();

  virtual string pluginName() const { return MSGXML_TYPENAME;};
  virtual string description() const { return "XML Message Engine";};
  virtual unsigned int vmajor() const { return MSGXML_PLUGIN_VMAJOR;};
  virtual unsigned int vminor() const { return MSGXML_PLUGIN_VMINOR;};

  /**
   * This method should only be used if there is no configuration file.
   * It creates a medium with default settings.
   */
  virtual Pointer<MessageEngine> engineFactory(int country,
					       const string &bankCode,
					       int hversion);

  /**
   * This method is used to create a medium object using the data given in
   * a configuration file.
   * There are some mandatory variables in this group:
   * <ul>
   *  <li>
   *   mediumtype: This is used to lookup the medium plugin that is
   *   responsible for this medium
   *  </li>
   *  <li>
   *   mediumname: This is used to identify a medium. Each medium MUST have
   *   a unique name. RDHFiles use the path and name of the keyfile here,
   *   while chipcard media use the card's serial number.
   *  </li>
   * </ul>
   * The plugin may additionally use whichever variables it finds necessary.
   */
  virtual Pointer<MessageEngine> engineFromConfig(int country,
						  const string &bankCode,
                                                  int hversion,
						  GWEN_DB_NODE *cfg);

  /**
   * This method is used to store information from a medium to a configuration
   * file (in order to be restorable by @ref mediumFromConfig).
   * See @ref mediumFromConfig for details about the config group.
   */
  virtual Error engineToConfig(Pointer<MessageEngine> e,
			       GWEN_DB_NODE *cfg);

};

}; // namespace

#endif // __cplusplus

#endif



