/***************************************************************************
 $RCSfile: msgxmlqueue.cpp,v $
                             -------------------
    cvs         : $Id: msgxmlqueue.cpp,v 1.3 2004/01/19 18:53:50 aquamaniac Exp $
    begin       : Sun Jul 06 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : openhbci@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "msgxmlqueue.h"
#include <gwenhywfar/text.h>


namespace HBCI {

  MsgQueueXML::MsgQueueXML(Pointer<Customer> cu,
                           int maxdiffjobs)
  :MsgQueue()
  , _msgnum(1)
  ,_dialogid("0")
  ,_owner(cu)
  ,_maxdiffjobs(maxdiffjobs){

    _cfg=GWEN_DB_Group_new("MsgQueueXML::_cfg");

  }


  MsgQueueXML::~MsgQueueXML(){
    GWEN_DB_Group_free(_cfg);
  }


  void MsgQueueXML::nextMessage(){
  }


  string MsgQueueXML::getProperty(const string &name,
				  const string &defValue){
    return GWEN_DB_GetCharValue(_cfg, name.c_str(), 0, defValue.c_str());
  }


  int MsgQueueXML::getIntProperty(const string &name,
				  int defValue){
    return GWEN_DB_GetIntValue(_cfg, name.c_str(), 0, defValue);
  }


  string MsgQueueXML::getBinProperty(const string &name,
				     const string &defValue){
    const void *d;
    unsigned int retsize;

    d=GWEN_DB_GetBinValue(_cfg, name.c_str(),
                          0,
                          defValue.data(),
                          defValue.length(),
                          &retsize);
    if (d)
      return string((char*)d, retsize);

    return defValue;
  }

  void MsgQueueXML::setBinProperty(const string &name, const string &value){
    if (GWEN_DB_SetBinValue(_cfg,
                            GWEN_DB_FLAGS_DEFAULT |
                            GWEN_DB_FLAGS_OVERWRITE_VARS,
                            name.c_str(),
                            value.c_str(),
                            value.length())) {
      throw Error("MsgJobXML::setBinPropertyt()",
                  ERROR_LEVEL_NORMAL,
                  HBCI_ERROR_CODE_UNKNOWN,
                  ERROR_ADVISE_DONTKNOW,
                  "Error setting binary value.",
                  name);
    }
  }


  void MsgQueueXML::setIntProperty(const string &name, int value){
    GWEN_DB_SetIntValue(_cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_DB_FLAGS_OVERWRITE_VARS,
                        name.c_str(), value);
  }


  void MsgQueueXML::setProperty(const string &name, const string &value){
    GWEN_DB_SetCharValue(_cfg,
                         GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         name.c_str(), value.c_str());
  }


  GWEN_DB_NODE *MsgQueueXML::queueData() {
    if (_cfg==0)
      _cfg=GWEN_DB_Group_new("MsgQueueXML::_cfg");
    return _cfg;
  }


  GWEN_DB_NODE *MsgQueueXML::queueResponse() {
    return GWEN_DB_Group_dup(queueData());
  }


}; /* namespace */




