/***************************************************************************
  $RCSfile: swiftplugin.cpp,v $
  -------------------
  cvs         : $Id: swiftplugin.cpp,v 1.5 2004/01/15 21:39:55 aquamaniac Exp $
  begin       : Mon Jul 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "swiftplugin.h"
#include "swiftparser.h"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>


namespace HBCI {

  SwiftParserPlugin::SwiftParserPlugin(Hbci *hbci)
  :ParserPlugin(hbci){
  }



  SwiftParserPlugin::~SwiftParserPlugin(){
  }



  int SwiftParserPlugin::readBinType(GWEN_XMLNODE *node,
                                     GWEN_DB_NODE *gr,
                                     GWEN_BUFFER *gbuf) {
    const char *type;
    const char *subtype;
    string datastr;
    unsigned int pos;
    GWEN_DB_NODE *mtgrp;
    GWEN_DB_NODE *reportgr;
    const char *name;

    name=GWEN_XMLNode_GetProperty(node, "name","");
    type=GWEN_XMLNode_GetProperty(node, "bintype","");
    subtype=GWEN_XMLNode_GetProperty(node, "binsubtype","");
    if (strcasecmp(type, "swift")) {
      DBG_ERROR(0,"Unsupported type (\"%s\" is not SWIFT)", type);
      return 1;
    }
    if (strcasecmp(subtype, "mt940")==0) {
      DBG_INFO(0, "Parsing MT940");
      datastr=string(GWEN_Buffer_GetStart(gbuf),
                     GWEN_Buffer_GetUsedBytes(gbuf));

      pos=0;
      mtgrp=GWEN_DB_GetGroup(gr, GWEN_DB_FLAGS_DEFAULT, name);
      assert(mtgrp);
      while (pos<datastr.length()) {
        reportgr=GWEN_DB_GetGroup(mtgrp,
                                  GWEN_PATH_FLAGS_CREATE_GROUP,
                                  "report");
        assert(reportgr);
        if (!SWIFTparser::readMT940(datastr, reportgr, pos)) {
          DBG_NOTICE(0, "Error parsing SWIFT MT940 structure");
          break;
        }
        DBG_INFO(0, "Added day report");
      } // while
    } // mt940
    else if (strcasecmp(subtype, "mt942")==0) {
      DBG_INFO(0, "Parsing MT942");
      datastr=string(GWEN_Buffer_GetStart(gbuf),
                     GWEN_Buffer_GetUsedBytes(gbuf));

      pos=0;
      mtgrp=GWEN_DB_GetGroup(gr, GWEN_DB_FLAGS_DEFAULT, name);
      assert(mtgrp);
      while (pos<datastr.length()) {
        reportgr=GWEN_DB_GetGroup(mtgrp,
                                  GWEN_PATH_FLAGS_CREATE_GROUP,
                                  "report");
        assert(reportgr);
        if (!SWIFTparser::readMT942(datastr, reportgr, pos)) {
          DBG_NOTICE(0, "Error parsing SWIFT MT942 structure");
          break;
        }
        DBG_INFO(0, "Added day report");
      } // while
    } // mt942
    else {
      DBG_ERROR(0,"Unsupported suptype \"%s\"",subtype);
      return -1;
    }

    return 0;
  }



  int SwiftParserPlugin::writeBinType(GWEN_XMLNODE *node,
                                      GWEN_DB_NODE *gr,
                                      GWEN_BUFFER *gbuf) {
    DBG_ERROR(0,"SwiftParserPlugin::writeBinType not implemented");
    return -1;
  }


}; // namespace




/* Plugin handling */

HBCI::Error swift_checkVersion() {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  HBCI::Hbci::libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile SWIFT plugin.\n");
    return HBCI::Error("SWIFT Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v%d.%d.%d.%d or newer.\n",
	    OPENHBCI_VERSION_MAJOR,
	    OPENHBCI_VERSION_MINOR,
	    OPENHBCI_VERSION_PATCHLEVEL,
	    OPENHBCI_VERSION_BUILD);
    return HBCI::Error("SWIFT Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }
  return HBCI::Error();
}

extern "C" {
  // The version of openhbci's plugin interface system (i.e. the
  // convention that this plugin has to define a registerYourSelf()
  // function.)
  int swift_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  // Version numbers of this plugin
  int swift_openhbci_module_current=SWIFT_MODULE_CURRENT;
  int swift_openhbci_module_revision=SWIFT_MODULE_REVISION;
  int swift_openhbci_module_age=SWIFT_MODULE_AGE;


  // The function that registers this plugin at the HBCI::API. This
  // function will be looked up by dlsym() and called from the plugin
  // interface system.
  HBCI::Error swift_registerPlugin(HBCI::Hbci *hbci,
				   HBCI::PluginManager *mgr,
				   PluginType pt) {
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    if (pt!=PluginTypeParser) {
      return HBCI::Error("SWIFT Plugin",
			 ERROR_LEVEL_NORMAL,
			 HBCI_ERROR_CODE_MEDIUM,
			 ERROR_ADVISE_DONTKNOW,
			 "bad plugin type");
    }
    err=swift_checkVersion();
    if (!err.isOk())
      return HBCI::Error("SWIFT Plugin",err);

    mp=new HBCI::SwiftParserPlugin(hbci);
    return mgr->registerPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::Plugin> swift_createPlugin(HBCI::Hbci *hbci,
						 HBCI::PluginManager *mgr){
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=swift_checkVersion();
    if (!err.isOk())
      throw HBCI::Error("SWIFT Plugin",err);

    mp=new HBCI::SwiftParserPlugin(hbci);
    mp.setDescription("SWIFTPlugin");
    return mp;
  }
} // extern "C"








