/***************************************************************************
  $RCSfile: xptcpplugin.cpp,v $
  -------------------
  cvs         : $Id: xptcpplugin.cpp,v 1.2 2004/01/19 20:20:36 aquamaniac Exp $
  begin       : Mon Apr 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "xptcpplugin.h"
#include "hbcistring.h"
#include <assert.h>
#include <openhbci2/pluginmanager.h>
#include <openhbci2/hbci.h>


namespace HBCI {


  TransporterPluginTCP::TransporterPluginTCP(Hbci *hbci)
  :TransporterPlugin(hbci)
  {
  }


  TransporterPluginTCP::~TransporterPluginTCP(){
  }

  string TransporterPluginTCP::description() const{
    return "Transportlevel plugin for TCP";
  }


  Pointer<Transporter>
  TransporterPluginTCP::transporterFactory(const string &addr,
					   const string &port,
					   int timeout){
    return new TransporterTCP(addr, port, timeout);
  }


  Pointer<Transporter>
    TransporterPluginTCP::transporterFromConfig(const string &address,
						const string &port,
						GWEN_DB_NODE *cfg){
    string typ;
    Pointer<Transporter> t;
    int timeout;

    if (!cfg) {
      throw Error("TransporterPluginTCP::engineFromConfig",
		  ERROR_LEVEL_NORMAL,
		  HBCI_ERROR_CODE_INVALID,
		  ERROR_ADVISE_DONTKNOW,
		  "no configuration given");
    }
    typ=GWEN_DB_GetCharValue(cfg, "transporttype",0, "");
    if (strcasecmp(typ.c_str(), pluginName().c_str())!=0) {
      throw Error("TransporterPluginTCP::transporterFromConfig",
		  ERROR_LEVEL_NORMAL,
		  HBCI_ERROR_CODE_INVALID,
		  ERROR_ADVISE_DONTKNOW,
		  "bad transporter type");
    }
    timeout=GWEN_DB_GetIntValue(cfg, "timeout",0, 60);
    t=new TransporterTCP(address, port, timeout);
    t.setObjectDescription("TransporterPluginTCP::transporterFromConfig");
    t.setDescription("TransporterPluginTCP::transporterFromConfig");
    return t;

  }


  Error TransporterPluginTCP::transporterToConfig(Pointer<Transporter> t,
						  GWEN_DB_NODE *cfg){
    Pointer<TransporterTCP> mkf;

    try {
      mkf=t.cast<TransporterTCP>();
    }
    catch (Error xerr) {
      return Error("MsgXMLPlugin::engineToConfig",xerr);
    }
    GWEN_DB_SetCharValue(cfg,
                         GWEN_DB_FLAGS_DEFAULT |
                         GWEN_DB_FLAGS_OVERWRITE_VARS,
                         "transporttype",
                         pluginName().c_str());
    GWEN_DB_SetIntValue(cfg,
                        GWEN_DB_FLAGS_DEFAULT |
                        GWEN_DB_FLAGS_OVERWRITE_VARS,
                        "timeout",
                        mkf.ref().timeout());
    return Error();
  }


}; // namespace


HBCI::Error xptcp_checkVersion() {
  int vmajor, vminor, vpl, vbuild;

  /* do some version checking
   * Please note that the constants used below are defined upon
   * compilation of this plugin. If you compile this plugin using
   * OpenHBCI 0.9.10 and try to use it on a system where OpenHBCI 0.9.9
   * is installed, this module will refuse to work. This plugin only
   * works with versions of OpenHBCI which are the same or newer than
   * the version used to compile this plugin.
   * If the major version differs then this module believes that the
   * version of OpenHBCI used is totally incompatible with the version
   * used to build this plugin.
   */
  HBCI::Hbci::libraryVersion(vmajor, vminor, vpl, vbuild);
  if (vmajor!=OPENHBCI_VERSION_MAJOR) {
    /* this is the major compatibility check. The major version is
     * incremented if a newer version is incompatible with older ones */
    fprintf(stderr,
	    " Different major versions, "
	    "please recompile RDHFile plugin.\n");
    return HBCI::Error("TransporterTCP Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "Major version does not match");
  }
  if (!(
	((vminor==OPENHBCI_VERSION_MINOR)&&(vpl>OPENHBCI_VERSION_PATCHLEVEL))
	||
	((vminor==OPENHBCI_VERSION_MINOR) &&
	 (vpl==OPENHBCI_VERSION_PATCHLEVEL) &&
	 (vbuild>=OPENHBCI_VERSION_BUILD))
       )
     ) {
    fprintf(stderr,
	    "This plugin needs OpenHBCI v%d.%d.%d.%d or newer.\n",
	    OPENHBCI_VERSION_MAJOR,
	    OPENHBCI_VERSION_MINOR,
	    OPENHBCI_VERSION_PATCHLEVEL,
	    OPENHBCI_VERSION_BUILD);
    return HBCI::Error("TransporterTCP Plugin",
		       ERROR_LEVEL_NORMAL,
		       HBCI_ERROR_CODE_MEDIUM,
		       ERROR_ADVISE_DONTKNOW,
		       "need OpenHBCI v"OPENHBCI_VERSION_FULL_STRING
		       " or newer");
  }
  return HBCI::Error();
}


extern "C" {
  // The version of openhbci's plugin interface system (i.e. the
  // convention that this plugin has to define a registerYourSelf()
  // function.)
  int xptcp_openhbci_plugin_interface_version = OPENHBCI_PLUGIN_INTERFACE_VERSION;

  // Version numbers of this plugin
  int xptcp_openhbci_module_current=XPTCP_MODULE_CURRENT;
  int xptcp_openhbci_module_revision=XPTCP_MODULE_REVISION;
  int xptcp_openhbci_module_age=XPTCP_MODULE_AGE;


  // The function that registers this plugin at the HBCI::API. This
  // function will be looked up by dlsym() and called from the plugin
  // interface system.
  HBCI::Error xptcp_registerPlugin(HBCI::Hbci *hbci,
				   HBCI::PluginManager *mgr,
				   PluginType pt) {
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    if (pt!=PluginTypeTransport) {
      return HBCI::Error("TransporterTCP Plugin",
			 ERROR_LEVEL_NORMAL,
			 HBCI_ERROR_CODE_MEDIUM,
			 ERROR_ADVISE_DONTKNOW,
			 "bad plugin type");
    }
    err=xptcp_checkVersion();
    if (!err.isOk())
      return HBCI::Error("TransporterTCP Plugin",err);

    mp=new HBCI::TransporterPluginTCP(hbci);
    return mgr->registerPlugin(mp);
  }

  /*
   * This function simply creates a medium plugin without registering it
   */
  HBCI::Pointer<HBCI::Plugin> xptcp_createPlugin(HBCI::Hbci *hbci,
						 HBCI::PluginManager *mgr){
    HBCI::Pointer<HBCI::Plugin> mp;
    HBCI::Error err;

    err=xptcp_checkVersion();
    if (!err.isOk())
      throw HBCI::Error("TransporterTCP Plugin",err);

    mp=new HBCI::TransporterPluginTCP(hbci);
    mp.setDescription("TransporterPluginTCP");
    return mp;
  }


}; // extern "C"




