/***************************************************************************
  $RCSfile: xptcpplugin.h,v $
  -------------------
  cvs         : $Id: xptcpplugin.h,v 1.2 2004/01/19 20:20:36 aquamaniac Exp $
  begin       : Mon Apr 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/
/**
 * @file mediumplugin.h Declaration of the abstract base class for a
 * medium plugin.
 */

#ifndef TRANSPORTTCPPLUGIN_H
#define TRANSPORTTCPPLUGIN_H

#define XPTCP_VERSION_MAJOR 1
#define XPTCP_VERSION_MINOR 0

#define XPTCP_MODULE_CURRENT  0
#define XPTCP_MODULE_REVISION 0
#define XPTCP_MODULE_AGE      0


#include <openhbci2/transporterplugin.h>
#include "xptcp.h"


#ifdef __cplusplus
#include <string>
using namespace std;

namespace HBCI {

/**
 * @brief This is the abstract base class for a transporter plugin.
 *
 * Note: If you are writing a new transporter plugin, then the *first*
 * class your new transporter class has to inherit is "HBCI::Transporter" (not
 * the other way round)!
 *
 * @author Martin Preuss<martin@libchipcard.de>
 * @see @ref pluginpage
 */
class TransporterPluginTCP: public TransporterPlugin {
public:
  /** Default Constructor */
  TransporterPluginTCP(Hbci *hbci);
  /** Default Destructor */
  virtual ~TransporterPluginTCP();

  virtual string pluginName() const { return XPTCP_TYPENAME;};
  /** Returns the human-readable description of this plugin. */
  virtual string description() const;
  virtual unsigned int vmajor() const { return XPTCP_VERSION_MAJOR;};
  virtual unsigned int vminor() const { return XPTCP_VERSION_MINOR;};

  /**
   * Returns the name of the TransportType (such as "XPTCP").
   */
  virtual string transporterTypeName() const { return XPTCP_TYPENAME; };

  /**
   */
  virtual Pointer<Transporter> transporterFactory(const string &addr,
						  const string &port,
						  int timeout);

  /**
   * This method is used to create a medium object using the data given in
   * a configuration file.
   * There are some mandatory variables in this group:
   * <ul>
   *  <li>
   *   transporttype: This is used to lookup the medium plugin that is
   *   responsible for this medium
   *  </li>
   * </ul>
   * The plugin may additionally use whichever variables it finds necessary.
   */
  virtual Pointer<Transporter> transporterFromConfig(const string &address,
						     const string &port,
                                                     GWEN_DB_NODE *cfg);
  /**
   * This method is used to store information from a transporter to a
   * configuration
   * file (in order to be restorable by @ref mediumFromConfig).
   * See @ref mediumFromConfig for details about the config group.
   */
  virtual Error transporterToConfig(Pointer<Transporter> m,
				    GWEN_DB_NODE *cfg);

};


}; /* namespace */
#endif /* __cplusplus */

#endif

